# Generated from english.sbl by Snowball 3.1.1 - https://snowballstem.org/

from .basestemmer import BaseStemmer
from .among import Among


class EnglishStemmer(BaseStemmer):
    '''
    This class implements the stemming algorithm defined by a snowball script.
    Generated from english.sbl by Snowball 3.1.1 - https://snowballstem.org/
    '''

    g_aeo = "aeo"
    g_v = {"a", "e", "i", "o", "u", "y"}

    g_v_WXY = {"Y", "a", "e", "i", "o", "u", "w", "x", "y"}

    g_valid_LI = {"c", "d", "e", "g", "h", "k", "m", "n", "r", "t"}

    B_Y_found = False
    I_p2 = 0
    I_p1 = 0

    def __r_prelude(self):
        self.B_Y_found = False
        v_1 = self.cursor
        try:
            self.bra = self.cursor
            if self.cursor == self.limit or self.current[self.cursor] != "'":
                raise lab0()
            self.cursor += 1
            self.ket = self.cursor
            self.slice_del()
        except lab0: pass
        self.cursor = v_1
        v_2 = self.cursor
        try:
            self.bra = self.cursor
            if self.cursor == self.limit or self.current[self.cursor] != "y":
                raise lab0()
            self.cursor += 1
            self.ket = self.cursor
            self.slice_from("Y")
            self.B_Y_found = True
        except lab0: pass
        self.cursor = v_2
        v_3 = self.cursor
        try:
            while True:
                v_4 = self.cursor
                try:
                    while True:
                        v_5 = self.cursor
                        try:
                            if not self.in_grouping(EnglishStemmer.g_v):
                                raise lab2()
                            self.bra = self.cursor
                            if self.cursor == self.limit or self.current[self.cursor] != "y":
                                raise lab2()
                            self.cursor += 1
                            self.ket = self.cursor
                            self.cursor = v_5
                            break
                        except lab2: pass
                        self.cursor = v_5
                        if self.cursor >= self.limit:
                            raise lab1()
                        self.cursor += 1
                    self.slice_from("Y")
                    self.B_Y_found = True
                    continue
                except lab1: pass
                self.cursor = v_4
                break
        except lab0: pass
        self.cursor = v_3
        return True

    def __r_mark_regions(self):
        self.I_p1 = self.limit
        self.I_p2 = self.limit
        v_1 = self.cursor
        try:
            while True:
                v_2 = self.cursor
                try:
                    if self.find_among(EnglishStemmer.a_0) == 0:
                        raise lab1()
                    break
                except lab1: pass
                self.cursor = v_2
                if not self.go_out_grouping(EnglishStemmer.g_v):
                    raise lab0()
                self.cursor += 1
                if not self.go_in_grouping(EnglishStemmer.g_v):
                    raise lab0()
                self.cursor += 1
                break
            self.I_p1 = self.cursor
            if not self.go_out_grouping(EnglishStemmer.g_v):
                raise lab0()
            self.cursor += 1
            if not self.go_in_grouping(EnglishStemmer.g_v):
                raise lab0()
            self.cursor += 1
            self.I_p2 = self.cursor
        except lab0: pass
        self.cursor = v_1
        return True

    def __r_shortv(self):
        while True:
            v_1 = self.limit - self.cursor
            try:
                if not self.out_grouping_b(EnglishStemmer.g_v_WXY):
                    raise lab0()
                if not self.in_grouping_b(EnglishStemmer.g_v):
                    raise lab0()
                if not self.out_grouping_b(EnglishStemmer.g_v):
                    raise lab0()
                break
            except lab0: pass
            self.cursor = self.limit - v_1
            try:
                if not self.out_grouping_b(EnglishStemmer.g_v):
                    raise lab0()
                if not self.in_grouping_b(EnglishStemmer.g_v):
                    raise lab0()
                if self.cursor > self.limit_backward:
                    raise lab0()
                break
            except lab0: pass
            self.cursor = self.limit - v_1
            if not self.eq_s_b("past"):
                return False
            break
        return True

    def __r_R1(self):
        return self.I_p1 <= self.cursor

    def __r_R2(self):
        return self.I_p2 <= self.cursor

    def __r_Step_1a(self):
        v_1 = self.limit - self.cursor
        try:
            self.ket = self.cursor
            if self.find_among_b(EnglishStemmer.a_1) == 0:
                self.cursor = self.limit - v_1
                raise lab0()
            self.bra = self.cursor
            self.slice_del()
        except lab0: pass
        self.ket = self.cursor
        among_var = self.find_among_b(EnglishStemmer.a_2)
        if among_var == 0:
            return False
        self.bra = self.cursor
        if among_var == 1:
            self.slice_from("ss")
        elif among_var == 2:
            while True:
                v_2 = self.limit - self.cursor
                try:
                    if self.cursor - 2 < self.limit_backward:
                        raise lab0()
                    self.cursor -= 2
                    self.slice_from("i")
                    break
                except lab0: pass
                self.cursor = self.limit - v_2
                self.slice_from("ie")
                break
        elif among_var == 3:
            if self.cursor <= self.limit_backward:
                return False
            self.cursor -= 1
            if not self.go_out_grouping_b(EnglishStemmer.g_v):
                return False
            self.cursor -= 1
            self.slice_del()
        return True

    def __r_Step_1b(self):
        self.ket = self.cursor
        among_var = self.find_among_b(EnglishStemmer.a_5)
        self.bra = self.cursor
        while True:
            v_1 = self.limit - self.cursor
            try:
                if among_var == 1:
                    v_2 = self.limit - self.cursor
                    try:
                        if not self.__r_R1():
                            raise lab1()
                        while True:
                            v_3 = self.limit - self.cursor
                            try:
                                if self.find_among_b(EnglishStemmer.a_3) == 0:
                                    raise lab2()
                                if self.cursor > self.limit_backward:
                                    raise lab2()
                                break
                            except lab2: pass
                            self.cursor = self.limit - v_3
                            self.slice_from("ee")
                            break
                    except lab1: pass
                    self.cursor = self.limit - v_2
                elif among_var == 2:
                    raise lab0()
                elif among_var == 3:
                    among_var = self.find_among_b(EnglishStemmer.a_4)
                    if among_var == 0:
                        raise lab0()
                    if among_var == 1:
                        v_4 = self.limit - self.cursor
                        if not self.out_grouping_b(EnglishStemmer.g_v):
                            raise lab0()
                        if self.cursor > self.limit_backward:
                            raise lab0()
                        self.cursor = self.limit - v_4
                        self.bra = self.cursor
                        self.slice_from("ie")
                    else:
                        if self.cursor > self.limit_backward:
                            raise lab0()
                break
            except lab0: pass
            self.cursor = self.limit - v_1
            v_5 = self.limit - self.cursor
            if not self.go_out_grouping_b(EnglishStemmer.g_v):
                return False
            self.cursor -= 1
            self.cursor = self.limit - v_5
            self.slice_del()
            self.ket = self.cursor
            self.bra = self.cursor
            v_6 = self.limit - self.cursor
            among_var = self.find_among_b(EnglishStemmer.a_6)
            if among_var == 1:
                self.slice_from("e")
                return False
            elif among_var == 2:
                v_7 = self.limit - self.cursor
                try:
                    if not self.in_grouping_b(EnglishStemmer.g_aeo):
                        raise lab0()
                    if self.cursor > self.limit_backward:
                        raise lab0()
                    return False
                except lab0: pass
                self.cursor = self.limit - v_7
            else:
                if self.cursor != self.I_p1:
                    return False
                v_8 = self.limit - self.cursor
                if not self.__r_shortv():
                    return False
                self.cursor = self.limit - v_8
                self.slice_from("e")
                return False
            self.cursor = self.limit - v_6
            self.ket = self.cursor
            if self.cursor <= self.limit_backward:
                return False
            self.cursor -= 1
            self.bra = self.cursor
            self.slice_del()
            break
        return True

    def __r_Step_1c(self):
        self.ket = self.cursor
        while True:
            try:
                if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "y":
                    raise lab0()
                self.cursor -= 1
                break
            except lab0: pass
            if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "Y":
                return False
            self.cursor -= 1
            break
        self.bra = self.cursor
        if not self.out_grouping_b(EnglishStemmer.g_v):
            return False
        if self.cursor <= self.limit_backward:
            return False
        self.slice_from("i")
        return True

    def __r_Step_2(self):
        self.ket = self.cursor
        among_var = self.find_among_b(EnglishStemmer.a_7)
        if among_var == 0:
            return False
        self.bra = self.cursor
        if not self.__r_R1():
            return False
        if among_var == 1:
            self.slice_from("tion")
        elif among_var == 2:
            self.slice_from("ence")
        elif among_var == 3:
            self.slice_from("ance")
        elif among_var == 4:
            self.slice_from("able")
        elif among_var == 5:
            self.slice_from("ent")
        elif among_var == 6:
            self.slice_from("ize")
        elif among_var == 7:
            self.slice_from("ate")
        elif among_var == 8:
            self.slice_from("al")
        elif among_var == 9:
            self.slice_from("ful")
        elif among_var == 10:
            self.slice_from("ous")
        elif among_var == 11:
            self.slice_from("ive")
        elif among_var == 12:
            self.slice_from("ble")
        elif among_var == 13:
            self.slice_from("og")
        elif among_var == 14:
            if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "l":
                return False
            self.cursor -= 1
            self.slice_from("og")
        elif among_var == 15:
            self.slice_from("less")
        else:
            if not self.in_grouping_b(EnglishStemmer.g_valid_LI):
                return False
            self.slice_del()
        return True

    def __r_Step_3(self):
        self.ket = self.cursor
        among_var = self.find_among_b(EnglishStemmer.a_8)
        if among_var == 0:
            return False
        self.bra = self.cursor
        if not self.__r_R1():
            return False
        if among_var == 1:
            self.slice_from("tion")
        elif among_var == 2:
            self.slice_from("ate")
        elif among_var == 3:
            self.slice_from("al")
        elif among_var == 4:
            self.slice_from("ic")
        elif among_var == 5:
            self.slice_del()
        else:
            if not self.__r_R2():
                return False
            self.slice_del()
        return True

    def __r_Step_4(self):
        self.ket = self.cursor
        among_var = self.find_among_b(EnglishStemmer.a_9)
        if among_var == 0:
            return False
        self.bra = self.cursor
        if not self.__r_R2():
            return False
        if among_var == 1:
            self.slice_del()
        else:
            while True:
                try:
                    if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "s":
                        raise lab0()
                    self.cursor -= 1
                    break
                except lab0: pass
                if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "t":
                    return False
                self.cursor -= 1
                break
            self.slice_del()
        return True

    def __r_Step_5(self):
        self.ket = self.cursor
        among_var = self.find_among_b(EnglishStemmer.a_10)
        if among_var == 0:
            return False
        self.bra = self.cursor
        if among_var == 1:
            while True:
                try:
                    if not self.__r_R2():
                        raise lab0()
                    break
                except lab0: pass
                if not self.__r_R1():
                    return False
                v_1 = self.limit - self.cursor
                try:
                    if not self.__r_shortv():
                        raise lab0()
                    return False
                except lab0: pass
                self.cursor = self.limit - v_1
                break
            self.slice_del()
        else:
            if not self.__r_R2():
                return False
            if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "l":
                return False
            self.cursor -= 1
            self.slice_del()
        return True

    def __r_exception1(self):
        self.bra = self.cursor
        among_var = self.find_among(EnglishStemmer.a_11)
        if among_var == 0:
            return False
        self.ket = self.cursor
        if self.cursor < self.limit:
            return False
        if among_var > 0:
            self.slice_from(EnglishStemmer.as_11[among_var - 1])
        return True

    def __r_postlude(self):
        if not self.B_Y_found:
            return False
        while True:
            v_1 = self.cursor
            try:
                while True:
                    v_2 = self.cursor
                    try:
                        self.bra = self.cursor
                        if self.cursor == self.limit or self.current[self.cursor] != "Y":
                            raise lab1()
                        self.cursor += 1
                        self.ket = self.cursor
                        self.cursor = v_2
                        break
                    except lab1: pass
                    self.cursor = v_2
                    if self.cursor >= self.limit:
                        raise lab0()
                    self.cursor += 1
                self.slice_from("y")
                continue
            except lab0: pass
            self.cursor = v_1
            break
        return True

    def _stem(self):
        while True:
            v_1 = self.cursor
            try:
                if not self.__r_exception1():
                    raise lab0()
                break
            except lab0: pass
            self.cursor = v_1
            try:
                try:
                    if self.cursor + 3 > self.limit:
                        raise lab1()
                    self.cursor += 3
                    raise lab0()
                except lab1: pass
                break
            except lab0: pass
            self.cursor = v_1
            self.__r_prelude()
            self.__r_mark_regions()
            self.limit_backward = self.cursor
            self.cursor = self.limit
            v_2 = self.limit - self.cursor
            self.__r_Step_1a()
            self.cursor = self.limit - v_2
            v_3 = self.limit - self.cursor
            self.__r_Step_1b()
            self.cursor = self.limit - v_3
            v_4 = self.limit - self.cursor
            self.__r_Step_1c()
            self.cursor = self.limit - v_4
            v_5 = self.limit - self.cursor
            self.__r_Step_2()
            self.cursor = self.limit - v_5
            v_6 = self.limit - self.cursor
            self.__r_Step_3()
            self.cursor = self.limit - v_6
            v_7 = self.limit - self.cursor
            self.__r_Step_4()
            self.cursor = self.limit - v_7
            v_8 = self.limit - self.cursor
            self.__r_Step_5()
            self.cursor = self.limit - v_8
            self.cursor = self.limit_backward
            v_9 = self.cursor
            self.__r_postlude()
            self.cursor = v_9
            break
        return True

    a_0 = [
        Among("arsen", -1, -1),
        Among("commun", -1, -1),
        Among("emerg", -1, -1),
        Among("gener", -1, -1),
        Among("inter", -1, -1),
        Among("later", -1, -1),
        Among("organ", -1, -1),
        Among("past", -1, -1),
        Among("univers", -1, -1)
    ]

    a_1 = [
        Among("'", -1, 1),
        Among("'s'", 0, 1),
        Among("'s", -1, 1)
    ]

    a_2 = [
        Among("ied", -1, 2),
        Among("s", -1, 3),
        Among("ies", 1, 2),
        Among("sses", 1, 1),
        Among("ss", 1, -1),
        Among("us", 1, -1)
    ]

    a_3 = [
        Among("succ", -1, 1),
        Among("proc", -1, 1),
        Among("exc", -1, 1)
    ]

    a_4 = [
        Among("even", -1, 2),
        Among("cann", -1, 2),
        Among("inn", -1, 2),
        Among("earr", -1, 2),
        Among("herr", -1, 2),
        Among("out", -1, 2),
        Among("y", -1, 1)
    ]

    a_5 = [
        Among("", -1, -1),
        Among("ed", 0, 2),
        Among("eed", 1, 1),
        Among("ing", 0, 3),
        Among("edly", 0, 2),
        Among("eedly", 4, 1),
        Among("ingly", 0, 2)
    ]

    a_6 = [
        Among("", -1, 3),
        Among("bb", 0, 2),
        Among("dd", 0, 2),
        Among("ff", 0, 2),
        Among("gg", 0, 2),
        Among("bl", 0, 1),
        Among("mm", 0, 2),
        Among("nn", 0, 2),
        Among("pp", 0, 2),
        Among("rr", 0, 2),
        Among("at", 0, 1),
        Among("tt", 0, 2),
        Among("iz", 0, 1)
    ]

    a_7 = [
        Among("anci", -1, 3),
        Among("enci", -1, 2),
        Among("ogi", -1, 14),
        Among("li", -1, 16),
        Among("bli", 3, 12),
        Among("abli", 4, 4),
        Among("alli", 3, 8),
        Among("fulli", 3, 9),
        Among("lessli", 3, 15),
        Among("ousli", 3, 10),
        Among("entli", 3, 5),
        Among("aliti", -1, 8),
        Among("biliti", -1, 12),
        Among("iviti", -1, 11),
        Among("tional", -1, 1),
        Among("ational", 14, 7),
        Among("alism", -1, 8),
        Among("ation", -1, 7),
        Among("ization", 17, 6),
        Among("izer", -1, 6),
        Among("ator", -1, 7),
        Among("iveness", -1, 11),
        Among("fulness", -1, 9),
        Among("ousness", -1, 10),
        Among("ogist", -1, 13)
    ]

    a_8 = [
        Among("icate", -1, 4),
        Among("ative", -1, 6),
        Among("alize", -1, 3),
        Among("iciti", -1, 4),
        Among("ical", -1, 4),
        Among("tional", -1, 1),
        Among("ational", 5, 2),
        Among("ful", -1, 5),
        Among("ness", -1, 5)
    ]

    a_9 = [
        Among("ic", -1, 1),
        Among("ance", -1, 1),
        Among("ence", -1, 1),
        Among("able", -1, 1),
        Among("ible", -1, 1),
        Among("ate", -1, 1),
        Among("ive", -1, 1),
        Among("ize", -1, 1),
        Among("iti", -1, 1),
        Among("al", -1, 1),
        Among("ism", -1, 1),
        Among("ion", -1, 2),
        Among("er", -1, 1),
        Among("ous", -1, 1),
        Among("ant", -1, 1),
        Among("ent", -1, 1),
        Among("ment", 15, 1),
        Among("ement", 16, 1)
    ]

    a_10 = [
        Among("e", -1, 1),
        Among("l", -1, 2)
    ]

    a_11 = [
        Among("andes", -1, -1),
        Among("atlas", -1, -1),
        Among("bias", -1, -1),
        Among("cosmos", -1, -1),
        Among("early", -1, 6),
        Among("gently", -1, 4),
        Among("howe", -1, -1),
        Among("idly", -1, 3),
        Among("news", -1, -1),
        Among("only", -1, 7),
        Among("singly", -1, 8),
        Among("skies", -1, 2),
        Among("skis", -1, 1),
        Among("sky", -1, -1),
        Among("ugly", -1, 5)
    ]
    as_11 = ("ski", "sky", "idl", "gentl", "ugli", "earli", "onli", "singl")


class lab0(BaseException): pass


class lab1(BaseException): pass


class lab2(BaseException): pass



# Generated from french.sbl by Snowball 3.1.1 - https://snowballstem.org/

from .basestemmer import BaseStemmer
from .among import Among


class FrenchStemmer(BaseStemmer):
    '''
    This class implements the stemming algorithm defined by a snowball script.
    Generated from french.sbl by Snowball 3.1.1 - https://snowballstem.org/
    '''

    g_v = {"a", "e", "i", "o", "u", "y", "à", "â", "è", "é", "ê", "ë", "î", "ï", "ô", "ù", "û"}

    g_oux_ending = {"b", "h", "j", "l", "n", "p"}

    g_elision_char = {"c", "d", "j", "l", "m", "n", "s", "t"}

    g_keep_with_s = {"a", "i", "o", "s", "u", "è"}

    I_p2 = 0
    I_p1 = 0
    I_pV = 0

    def __r_elisions(self):
        self.bra = self.cursor
        while True:
            try:
                if not self.in_grouping(FrenchStemmer.g_elision_char):
                    raise lab0()
                break
            except lab0: pass
            if not self.eq_s("qu"):
                return False
            break
        if self.cursor == self.limit or self.current[self.cursor] != "'":
            return False
        self.cursor += 1
        self.ket = self.cursor
        if self.cursor >= self.limit:
            return False
        self.slice_del()
        return True

    def __r_prelude(self):
        while True:
            v_1 = self.cursor
            try:
                while True:
                    v_2 = self.cursor
                    try:
                        while True:
                            v_3 = self.cursor
                            try:
                                if not self.in_grouping(FrenchStemmer.g_v):
                                    raise lab2()
                                self.bra = self.cursor
                                while True:
                                    v_4 = self.cursor
                                    try:
                                        if self.cursor == self.limit or self.current[self.cursor] != "u":
                                            raise lab3()
                                        self.cursor += 1
                                        self.ket = self.cursor
                                        if not self.in_grouping(FrenchStemmer.g_v):
                                            raise lab3()
                                        self.slice_from("U")
                                        break
                                    except lab3: pass
                                    self.cursor = v_4
                                    try:
                                        if self.cursor == self.limit or self.current[self.cursor] != "i":
                                            raise lab3()
                                        self.cursor += 1
                                        self.ket = self.cursor
                                        if not self.in_grouping(FrenchStemmer.g_v):
                                            raise lab3()
                                        self.slice_from("I")
                                        break
                                    except lab3: pass
                                    self.cursor = v_4
                                    if self.cursor == self.limit or self.current[self.cursor] != "y":
                                        raise lab2()
                                    self.cursor += 1
                                    self.ket = self.cursor
                                    self.slice_from("Y")
                                    break
                                break
                            except lab2: pass
                            self.cursor = v_3
                            try:
                                self.bra = self.cursor
                                if self.cursor == self.limit or self.current[self.cursor] != "ë":
                                    raise lab2()
                                self.cursor += 1
                                self.ket = self.cursor
                                self.slice_from("He")
                                break
                            except lab2: pass
                            self.cursor = v_3
                            try:
                                self.bra = self.cursor
                                if self.cursor == self.limit or self.current[self.cursor] != "ï":
                                    raise lab2()
                                self.cursor += 1
                                self.ket = self.cursor
                                self.slice_from("Hi")
                                break
                            except lab2: pass
                            self.cursor = v_3
                            try:
                                self.bra = self.cursor
                                if self.cursor == self.limit or self.current[self.cursor] != "y":
                                    raise lab2()
                                self.cursor += 1
                                self.ket = self.cursor
                                if not self.in_grouping(FrenchStemmer.g_v):
                                    raise lab2()
                                self.slice_from("Y")
                                break
                            except lab2: pass
                            self.cursor = v_3
                            if self.cursor == self.limit or self.current[self.cursor] != "q":
                                raise lab1()
                            self.cursor += 1
                            self.bra = self.cursor
                            if self.cursor == self.limit or self.current[self.cursor] != "u":
                                raise lab1()
                            self.cursor += 1
                            self.ket = self.cursor
                            self.slice_from("U")
                            break
                        self.cursor = v_2
                        break
                    except lab1: pass
                    self.cursor = v_2
                    if self.cursor >= self.limit:
                        raise lab0()
                    self.cursor += 1
                continue
            except lab0: pass
            self.cursor = v_1
            break
        return True

    def __r_mark_regions(self):
        self.I_pV = self.limit
        self.I_p1 = self.limit
        self.I_p2 = self.limit
        v_1 = self.cursor
        try:
            while True:
                v_2 = self.cursor
                try:
                    if not self.in_grouping(FrenchStemmer.g_v):
                        raise lab1()
                    if not self.in_grouping(FrenchStemmer.g_v):
                        raise lab1()
                    if self.cursor >= self.limit:
                        raise lab1()
                    self.cursor += 1
                    break
                except lab1: pass
                self.cursor = v_2
                try:
                    among_var = self.find_among(FrenchStemmer.a_0)
                    if among_var == 0:
                        raise lab1()
                    if among_var == 1:
                        if not self.in_grouping(FrenchStemmer.g_v):
                            raise lab1()
                    break
                except lab1: pass
                self.cursor = v_2
                if self.cursor >= self.limit:
                    raise lab0()
                self.cursor += 1
                if not self.go_out_grouping(FrenchStemmer.g_v):
                    raise lab0()
                self.cursor += 1
                break
            self.I_pV = self.cursor
        except lab0: pass
        self.cursor = v_1
        v_3 = self.cursor
        try:
            if not self.go_out_grouping(FrenchStemmer.g_v):
                raise lab0()
            self.cursor += 1
            if not self.go_in_grouping(FrenchStemmer.g_v):
                raise lab0()
            self.cursor += 1
            self.I_p1 = self.cursor
            if not self.go_out_grouping(FrenchStemmer.g_v):
                raise lab0()
            self.cursor += 1
            if not self.go_in_grouping(FrenchStemmer.g_v):
                raise lab0()
            self.cursor += 1
            self.I_p2 = self.cursor
        except lab0: pass
        self.cursor = v_3
        return True

    def __r_postlude(self):
        while True:
            v_1 = self.cursor
            try:
                self.bra = self.cursor
                among_var = self.find_among(FrenchStemmer.a_1)
                self.ket = self.cursor
                if among_var == 1:
                    self.slice_from("i")
                elif among_var == 2:
                    self.slice_from("u")
                elif among_var == 3:
                    self.slice_from("y")
                elif among_var == 4:
                    self.slice_from("ë")
                elif among_var == 5:
                    self.slice_from("ï")
                elif among_var == 6:
                    self.slice_del()
                else:
                    if self.cursor >= self.limit:
                        raise lab0()
                    self.cursor += 1
                continue
            except lab0: pass
            self.cursor = v_1
            break
        return True

    def __r_RV(self):
        return self.I_pV <= self.cursor

    def __r_R1(self):
        return self.I_p1 <= self.cursor

    def __r_R2(self):
        return self.I_p2 <= self.cursor

    def __r_standard_suffix(self):
        self.ket = self.cursor
        among_var = self.find_among_b(FrenchStemmer.a_4)
        if among_var == 0:
            return False
        self.bra = self.cursor
        if among_var == 1:
            if not self.__r_R2():
                return False
            self.slice_del()
        elif among_var == 2:
            if not self.__r_R2():
                return False
            self.slice_del()
            v_1 = self.limit - self.cursor
            try:
                self.ket = self.cursor
                if not self.eq_s_b("ic"):
                    self.cursor = self.limit - v_1
                    raise lab0()
                self.bra = self.cursor
                while True:
                    v_2 = self.limit - self.cursor
                    try:
                        if not self.__r_R2():
                            raise lab1()
                        self.slice_del()
                        break
                    except lab1: pass
                    self.cursor = self.limit - v_2
                    self.slice_from("iqU")
                    break
            except lab0: pass
        elif among_var == 3:
            if not self.__r_R2():
                return False
            self.slice_from("log")
        elif among_var == 4:
            if not self.__r_R2():
                return False
            self.slice_from("u")
        elif among_var == 5:
            if not self.__r_R2():
                return False
            self.slice_from("ent")
        elif among_var == 6:
            if not self.__r_RV():
                return False
            self.slice_del()
            v_3 = self.limit - self.cursor
            try:
                self.ket = self.cursor
                among_var = self.find_among_b(FrenchStemmer.a_2)
                if among_var == 0:
                    self.cursor = self.limit - v_3
                    raise lab0()
                self.bra = self.cursor
                if among_var == 1:
                    if not self.__r_R2():
                        self.cursor = self.limit - v_3
                        raise lab0()
                    self.slice_del()
                    self.ket = self.cursor
                    if not self.eq_s_b("at"):
                        self.cursor = self.limit - v_3
                        raise lab0()
                    self.bra = self.cursor
                    if not self.__r_R2():
                        self.cursor = self.limit - v_3
                        raise lab0()
                    self.slice_del()
                elif among_var == 2:
                    while True:
                        v_4 = self.limit - self.cursor
                        try:
                            if not self.__r_R2():
                                raise lab1()
                            self.slice_del()
                            break
                        except lab1: pass
                        self.cursor = self.limit - v_4
                        if not self.__r_R1():
                            self.cursor = self.limit - v_3
                            raise lab0()
                        self.slice_from("eux")
                        break
                elif among_var == 3:
                    if not self.__r_R2():
                        self.cursor = self.limit - v_3
                        raise lab0()
                    self.slice_del()
                else:
                    if not self.__r_RV():
                        self.cursor = self.limit - v_3
                        raise lab0()
                    self.slice_from("i")
            except lab0: pass
        elif among_var == 7:
            if not self.__r_R2():
                return False
            self.slice_del()
            v_5 = self.limit - self.cursor
            try:
                self.ket = self.cursor
                among_var = self.find_among_b(FrenchStemmer.a_3)
                if among_var == 0:
                    self.cursor = self.limit - v_5
                    raise lab0()
                self.bra = self.cursor
                if among_var == 1:
                    while True:
                        v_6 = self.limit - self.cursor
                        try:
                            if not self.__r_R2():
                                raise lab1()
                            self.slice_del()
                            break
                        except lab1: pass
                        self.cursor = self.limit - v_6
                        self.slice_from("abl")
                        break
                elif among_var == 2:
                    while True:
                        v_7 = self.limit - self.cursor
                        try:
                            if not self.__r_R2():
                                raise lab1()
                            self.slice_del()
                            break
                        except lab1: pass
                        self.cursor = self.limit - v_7
                        self.slice_from("iqU")
                        break
                else:
                    if not self.__r_R2():
                        self.cursor = self.limit - v_5
                        raise lab0()
                    self.slice_del()
            except lab0: pass
        elif among_var == 8:
            if not self.__r_R2():
                return False
            self.slice_del()
            v_8 = self.limit - self.cursor
            try:
                self.ket = self.cursor
                if not self.eq_s_b("at"):
                    self.cursor = self.limit - v_8
                    raise lab0()
                self.bra = self.cursor
                if not self.__r_R2():
                    self.cursor = self.limit - v_8
                    raise lab0()
                self.slice_del()
                self.ket = self.cursor
                if not self.eq_s_b("ic"):
                    self.cursor = self.limit - v_8
                    raise lab0()
                self.bra = self.cursor
                while True:
                    v_9 = self.limit - self.cursor
                    try:
                        if not self.__r_R2():
                            raise lab1()
                        self.slice_del()
                        break
                    except lab1: pass
                    self.cursor = self.limit - v_9
                    self.slice_from("iqU")
                    break
            except lab0: pass
        elif among_var == 9:
            self.slice_from("eau")
        elif among_var == 10:
            if not self.__r_R1():
                return False
            self.slice_from("al")
        elif among_var == 11:
            if not self.in_grouping_b(FrenchStemmer.g_oux_ending):
                return False
            self.slice_from("ou")
        elif among_var == 12:
            while True:
                v_10 = self.limit - self.cursor
                try:
                    if not self.__r_R2():
                        raise lab0()
                    self.slice_del()
                    break
                except lab0: pass
                self.cursor = self.limit - v_10
                if not self.__r_R1():
                    return False
                self.slice_from("eux")
                break
        elif among_var == 13:
            if not self.__r_R1():
                return False
            if not self.out_grouping_b(FrenchStemmer.g_v):
                return False
            self.slice_del()
        elif among_var == 14:
            if not self.__r_RV():
                return False
            self.slice_from("ant")
            return False
        elif among_var == 15:
            if not self.__r_RV():
                return False
            self.slice_from("ent")
            return False
        else:
            v_11 = self.limit - self.cursor
            if not self.in_grouping_b(FrenchStemmer.g_v):
                return False
            if not self.__r_RV():
                return False
            self.cursor = self.limit - v_11
            self.slice_del()
            return False
        return True

    def __r_i_verb_suffix(self):
        if self.cursor < self.I_pV:
            return False
        v_2 = self.limit_backward
        self.limit_backward = self.I_pV
        self.ket = self.cursor
        if self.find_among_b(FrenchStemmer.a_5) == 0:
            self.limit_backward = v_2
            return False
        self.bra = self.cursor
        try:
            if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "H":
                raise lab0()
            self.cursor -= 1
            self.limit_backward = v_2
            return False
        except lab0: pass
        if not self.out_grouping_b(FrenchStemmer.g_v):
            self.limit_backward = v_2
            return False
        self.slice_del()
        self.limit_backward = v_2
        return True

    def __r_verb_suffix(self):
        if self.cursor < self.I_pV:
            return False
        v_2 = self.limit_backward
        self.limit_backward = self.I_pV
        self.ket = self.cursor
        among_var = self.find_among_b(FrenchStemmer.a_7)
        if among_var == 0:
            self.limit_backward = v_2
            return False
        self.bra = self.cursor
        self.limit_backward = v_2
        if among_var == 1:
            if not self.__r_R2():
                return False
            self.slice_del()
        elif among_var == 2:
            self.slice_del()
        elif among_var == 3:
            v_3 = self.limit - self.cursor
            try:
                if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "e":
                    self.cursor = self.limit - v_3
                    raise lab0()
                self.cursor -= 1
                if not self.__r_RV():
                    self.cursor = self.limit - v_3
                    raise lab0()
                self.bra = self.cursor
            except lab0: pass
            self.slice_del()
        else:
            v_4 = self.limit - self.cursor
            try:
                among_var = self.find_among_b(FrenchStemmer.a_6)
                if among_var == 0:
                    raise lab0()
                if among_var == 1:
                    if self.cursor <= self.limit_backward:
                        raise lab0()
                    self.cursor -= 1
                    if self.cursor > self.limit_backward:
                        raise lab0()
                return False
            except lab0: pass
            self.cursor = self.limit - v_4
            self.slice_del()
        return True

    def __r_residual_suffix(self):
        v_1 = self.limit - self.cursor
        try:
            self.ket = self.cursor
            if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "s":
                self.cursor = self.limit - v_1
                raise lab0()
            self.cursor -= 1
            self.bra = self.cursor
            v_2 = self.limit - self.cursor
            while True:
                try:
                    if not self.eq_s_b("Hi"):
                        raise lab1()
                    break
                except lab1: pass
                if not self.out_grouping_b(FrenchStemmer.g_keep_with_s):
                    self.cursor = self.limit - v_1
                    raise lab0()
                break
            self.cursor = self.limit - v_2
            self.slice_del()
        except lab0: pass
        if self.cursor < self.I_pV:
            return False
        v_4 = self.limit_backward
        self.limit_backward = self.I_pV
        self.ket = self.cursor
        among_var = self.find_among_b(FrenchStemmer.a_8)
        if among_var == 0:
            self.limit_backward = v_4
            return False
        self.bra = self.cursor
        if among_var == 1:
            if not self.__r_R2():
                self.limit_backward = v_4
                return False
            while True:
                try:
                    if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "s":
                        raise lab0()
                    self.cursor -= 1
                    break
                except lab0: pass
                if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "t":
                    self.limit_backward = v_4
                    return False
                self.cursor -= 1
                break
            self.slice_del()
        elif among_var == 2:
            self.slice_from("i")
        else:
            self.slice_del()
        self.limit_backward = v_4
        return True

    def __r_un_double(self):
        v_1 = self.limit - self.cursor
        if self.find_among_b(FrenchStemmer.a_9) == 0:
            return False
        self.cursor = self.limit - v_1
        self.ket = self.cursor
        if self.cursor <= self.limit_backward:
            return False
        self.cursor -= 1
        self.bra = self.cursor
        self.slice_del()
        return True

    def __r_un_accent(self):
        v_1 = 1
        while True:
            try:
                if not self.out_grouping_b(FrenchStemmer.g_v):
                    raise lab0()
                v_1 -= 1
                continue
            except lab0: pass
            break
        if v_1 > 0:
            return False
        self.ket = self.cursor
        while True:
            try:
                if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "é":
                    raise lab0()
                self.cursor -= 1
                break
            except lab0: pass
            if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "è":
                return False
            self.cursor -= 1
            break
        self.bra = self.cursor
        self.slice_from("e")
        return True

    def _stem(self):
        v_1 = self.cursor
        self.__r_elisions()
        self.cursor = v_1
        v_2 = self.cursor
        self.__r_prelude()
        self.cursor = v_2
        self.__r_mark_regions()
        self.limit_backward = self.cursor
        self.cursor = self.limit
        v_3 = self.limit - self.cursor
        try:
            while True:
                v_4 = self.limit - self.cursor
                try:
                    v_5 = self.limit - self.cursor
                    while True:
                        v_6 = self.limit - self.cursor
                        try:
                            if not self.__r_standard_suffix():
                                raise lab2()
                            break
                        except lab2: pass
                        self.cursor = self.limit - v_6
                        try:
                            if not self.__r_i_verb_suffix():
                                raise lab2()
                            break
                        except lab2: pass
                        self.cursor = self.limit - v_6
                        if not self.__r_verb_suffix():
                            raise lab1()
                        break
                    self.cursor = self.limit - v_5
                    v_7 = self.limit - self.cursor
                    try:
                        self.ket = self.cursor
                        while True:
                            v_8 = self.limit - self.cursor
                            try:
                                if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "Y":
                                    raise lab3()
                                self.cursor -= 1
                                self.bra = self.cursor
                                self.slice_from("i")
                                break
                            except lab3: pass
                            self.cursor = self.limit - v_8
                            if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "ç":
                                self.cursor = self.limit - v_7
                                raise lab2()
                            self.cursor -= 1
                            self.bra = self.cursor
                            self.slice_from("c")
                            break
                    except lab2: pass
                    break
                except lab1: pass
                self.cursor = self.limit - v_4
                if not self.__r_residual_suffix():
                    raise lab0()
                break
        except lab0: pass
        self.cursor = self.limit - v_3
        v_9 = self.limit - self.cursor
        self.__r_un_double()
        self.cursor = self.limit - v_9
        v_10 = self.limit - self.cursor
        self.__r_un_accent()
        self.cursor = self.limit - v_10
        self.cursor = self.limit_backward
        v_11 = self.cursor
        self.__r_postlude()
        self.cursor = v_11
        return True

    a_0 = [
        Among("col", -1, -1),
        Among("ni", -1, 1),
        Among("par", -1, -1),
        Among("tap", -1, -1)
    ]

    a_1 = [
        Among("", -1, 7),
        Among("H", 0, 6),
        Among("He", 1, 4),
        Among("Hi", 1, 5),
        Among("I", 0, 1),
        Among("U", 0, 2),
        Among("Y", 0, 3)
    ]

    a_2 = [
        Among("iqU", -1, 3),
        Among("abl", -1, 3),
        Among("Ièr", -1, 4),
        Among("ièr", -1, 4),
        Among("eus", -1, 2),
        Among("iv", -1, 1)
    ]

    a_3 = [
        Among("ic", -1, 2),
        Among("abil", -1, 1),
        Among("iv", -1, 3)
    ]

    a_4 = [
        Among("iqUe", -1, 1),
        Among("atrice", -1, 2),
        Among("ance", -1, 1),
        Among("ence", -1, 5),
        Among("logie", -1, 3),
        Among("able", -1, 1),
        Among("isme", -1, 1),
        Among("euse", -1, 12),
        Among("iste", -1, 1),
        Among("ive", -1, 8),
        Among("if", -1, 8),
        Among("usion", -1, 4),
        Among("ation", -1, 2),
        Among("ution", -1, 4),
        Among("ateur", -1, 2),
        Among("iqUes", -1, 1),
        Among("atrices", -1, 2),
        Among("ances", -1, 1),
        Among("ences", -1, 5),
        Among("logies", -1, 3),
        Among("ables", -1, 1),
        Among("ismes", -1, 1),
        Among("euses", -1, 12),
        Among("istes", -1, 1),
        Among("ives", -1, 8),
        Among("ifs", -1, 8),
        Among("usions", -1, 4),
        Among("ations", -1, 2),
        Among("utions", -1, 4),
        Among("ateurs", -1, 2),
        Among("ments", -1, 16),
        Among("ements", 30, 6),
        Among("issements", 31, 13),
        Among("ités", -1, 7),
        Among("ment", -1, 16),
        Among("ement", 34, 6),
        Among("issement", 35, 13),
        Among("amment", 34, 14),
        Among("emment", 34, 15),
        Among("aux", -1, 10),
        Among("eaux", 39, 9),
        Among("eux", -1, 1),
        Among("oux", -1, 11),
        Among("ité", -1, 7)
    ]

    a_5 = [
        Among("ira", -1, 1),
        Among("ie", -1, 1),
        Among("isse", -1, 1),
        Among("issante", -1, 1),
        Among("i", -1, 1),
        Among("irai", 4, 1),
        Among("ir", -1, 1),
        Among("iras", -1, 1),
        Among("ies", -1, 1),
        Among("îmes", -1, 1),
        Among("isses", -1, 1),
        Among("issantes", -1, 1),
        Among("îtes", -1, 1),
        Among("is", -1, 1),
        Among("irais", 13, 1),
        Among("issais", 13, 1),
        Among("irions", -1, 1),
        Among("issions", -1, 1),
        Among("irons", -1, 1),
        Among("issons", -1, 1),
        Among("issants", -1, 1),
        Among("it", -1, 1),
        Among("irait", 21, 1),
        Among("issait", 21, 1),
        Among("issant", -1, 1),
        Among("iraIent", -1, 1),
        Among("issaIent", -1, 1),
        Among("irent", -1, 1),
        Among("issent", -1, 1),
        Among("iront", -1, 1),
        Among("ît", -1, 1),
        Among("iriez", -1, 1),
        Among("issiez", -1, 1),
        Among("irez", -1, 1),
        Among("issez", -1, 1)
    ]

    a_6 = [
        Among("al", -1, 1),
        Among("épl", -1, -1),
        Among("auv", -1, -1)
    ]

    a_7 = [
        Among("a", -1, 3),
        Among("era", 0, 2),
        Among("aise", -1, 4),
        Among("asse", -1, 3),
        Among("ante", -1, 3),
        Among("ée", -1, 2),
        Among("ai", -1, 3),
        Among("erai", 6, 2),
        Among("er", -1, 2),
        Among("as", -1, 3),
        Among("eras", 9, 2),
        Among("âmes", -1, 3),
        Among("aises", -1, 4),
        Among("asses", -1, 3),
        Among("antes", -1, 3),
        Among("âtes", -1, 3),
        Among("ées", -1, 2),
        Among("ais", -1, 4),
        Among("eais", 17, 2),
        Among("erais", 17, 2),
        Among("ions", -1, 1),
        Among("erions", 20, 2),
        Among("assions", 20, 3),
        Among("erons", -1, 2),
        Among("ants", -1, 3),
        Among("és", -1, 2),
        Among("ait", -1, 3),
        Among("erait", 26, 2),
        Among("ant", -1, 3),
        Among("aIent", -1, 3),
        Among("eraIent", 29, 2),
        Among("èrent", -1, 2),
        Among("assent", -1, 3),
        Among("eront", -1, 2),
        Among("ât", -1, 3),
        Among("ez", -1, 2),
        Among("iez", 35, 2),
        Among("eriez", 36, 2),
        Among("assiez", 36, 3),
        Among("erez", 35, 2),
        Among("é", -1, 2)
    ]

    a_8 = [
        Among("e", -1, 3),
        Among("Ière", 0, 2),
        Among("ière", 0, 2),
        Among("ion", -1, 1),
        Among("Ier", -1, 2),
        Among("ier", -1, 2)
    ]

    a_9 = [
        Among("ell", -1, -1),
        Among("eill", -1, -1),
        Among("enn", -1, -1),
        Among("onn", -1, -1),
        Among("ett", -1, -1)
    ]


class lab0(BaseException): pass


class lab1(BaseException): pass


class lab2(BaseException): pass


class lab3(BaseException): pass



# Generated from turkish.sbl by Snowball 3.1.1 - https://snowballstem.org/

from .basestemmer import BaseStemmer
from .among import Among


class TurkishStemmer(BaseStemmer):
    '''
    This class implements the stemming algorithm defined by a snowball script.
    Generated from turkish.sbl by Snowball 3.1.1 - https://snowballstem.org/
    '''

    g_vowel = {"a", "e", "i", "o", "u", "ö", "ü", "ı"}

    g_U = {"i", "u", "ü", "ı"}

    g_vowel1 = {"a", "o", "u", "ı"}

    g_vowel2 = {"e", "i", "ö", "ü"}

    g_vowel3 = "aı"
    g_vowel4 = "ei"
    g_vowel5 = "ou"
    g_vowel6 = "öü"
    B_continue_stemming_noun_suffixes = False

    def __r_check_vowel_harmony(self):
        v_1 = self.limit - self.cursor
        if not self.go_out_grouping_b(TurkishStemmer.g_vowel):
            return False
        while True:
            v_2 = self.limit - self.cursor
            try:
                if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "a":
                    raise lab0()
                self.cursor -= 1
                if not self.go_out_grouping_b(TurkishStemmer.g_vowel1):
                    raise lab0()
                break
            except lab0: pass
            self.cursor = self.limit - v_2
            try:
                if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "e":
                    raise lab0()
                self.cursor -= 1
                if not self.go_out_grouping_b(TurkishStemmer.g_vowel2):
                    raise lab0()
                break
            except lab0: pass
            self.cursor = self.limit - v_2
            try:
                if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "ı":
                    raise lab0()
                self.cursor -= 1
                if not self.go_out_grouping_b(TurkishStemmer.g_vowel3):
                    raise lab0()
                break
            except lab0: pass
            self.cursor = self.limit - v_2
            try:
                if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "i":
                    raise lab0()
                self.cursor -= 1
                if not self.go_out_grouping_b(TurkishStemmer.g_vowel4):
                    raise lab0()
                break
            except lab0: pass
            self.cursor = self.limit - v_2
            try:
                if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "o":
                    raise lab0()
                self.cursor -= 1
                if not self.go_out_grouping_b(TurkishStemmer.g_vowel5):
                    raise lab0()
                break
            except lab0: pass
            self.cursor = self.limit - v_2
            try:
                if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "ö":
                    raise lab0()
                self.cursor -= 1
                if not self.go_out_grouping_b(TurkishStemmer.g_vowel6):
                    raise lab0()
                break
            except lab0: pass
            self.cursor = self.limit - v_2
            try:
                if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "u":
                    raise lab0()
                self.cursor -= 1
                if not self.go_out_grouping_b(TurkishStemmer.g_vowel5):
                    raise lab0()
                break
            except lab0: pass
            self.cursor = self.limit - v_2
            if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "ü":
                return False
            self.cursor -= 1
            if not self.go_out_grouping_b(TurkishStemmer.g_vowel6):
                return False
            break
        self.cursor = self.limit - v_1
        return True

    def __r_mark_suffix_with_optional_n_consonant(self):
        while True:
            v_1 = self.limit - self.cursor
            try:
                if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "n":
                    raise lab0()
                self.cursor -= 1
                v_2 = self.limit - self.cursor
                if not self.in_grouping_b(TurkishStemmer.g_vowel):
                    raise lab0()
                self.cursor = self.limit - v_2
                break
            except lab0: pass
            self.cursor = self.limit - v_1
            try:
                if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "n":
                    raise lab0()
                self.cursor -= 1
                return False
            except lab0: pass
            v_3 = self.limit - self.cursor
            if self.cursor <= self.limit_backward:
                return False
            self.cursor -= 1
            if not self.in_grouping_b(TurkishStemmer.g_vowel):
                return False
            self.cursor = self.limit - v_3
            break
        return True

    def __r_mark_suffix_with_optional_s_consonant(self):
        while True:
            v_1 = self.limit - self.cursor
            try:
                if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "s":
                    raise lab0()
                self.cursor -= 1
                v_2 = self.limit - self.cursor
                if not self.in_grouping_b(TurkishStemmer.g_vowel):
                    raise lab0()
                self.cursor = self.limit - v_2
                break
            except lab0: pass
            self.cursor = self.limit - v_1
            try:
                if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "s":
                    raise lab0()
                self.cursor -= 1
                return False
            except lab0: pass
            v_3 = self.limit - self.cursor
            if self.cursor <= self.limit_backward:
                return False
            self.cursor -= 1
            if not self.in_grouping_b(TurkishStemmer.g_vowel):
                return False
            self.cursor = self.limit - v_3
            break
        return True

    def __r_mark_suffix_with_optional_y_consonant(self):
        while True:
            v_1 = self.limit - self.cursor
            try:
                if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "y":
                    raise lab0()
                self.cursor -= 1
                v_2 = self.limit - self.cursor
                if not self.in_grouping_b(TurkishStemmer.g_vowel):
                    raise lab0()
                self.cursor = self.limit - v_2
                break
            except lab0: pass
            self.cursor = self.limit - v_1
            try:
                if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "y":
                    raise lab0()
                self.cursor -= 1
                return False
            except lab0: pass
            v_3 = self.limit - self.cursor
            if self.cursor <= self.limit_backward:
                return False
            self.cursor -= 1
            if not self.in_grouping_b(TurkishStemmer.g_vowel):
                return False
            self.cursor = self.limit - v_3
            break
        return True

    def __r_mark_suffix_with_optional_U_vowel(self):
        while True:
            v_1 = self.limit - self.cursor
            try:
                if not self.in_grouping_b(TurkishStemmer.g_U):
                    raise lab0()
                v_2 = self.limit - self.cursor
                if not self.out_grouping_b(TurkishStemmer.g_vowel):
                    raise lab0()
                self.cursor = self.limit - v_2
                break
            except lab0: pass
            self.cursor = self.limit - v_1
            try:
                if not self.in_grouping_b(TurkishStemmer.g_U):
                    raise lab0()
                return False
            except lab0: pass
            v_3 = self.limit - self.cursor
            if self.cursor <= self.limit_backward:
                return False
            self.cursor -= 1
            if not self.out_grouping_b(TurkishStemmer.g_vowel):
                return False
            self.cursor = self.limit - v_3
            break
        return True

    def __r_mark_possessives(self):
        if self.find_among_b(TurkishStemmer.a_0) == 0:
            return False
        return self.__r_mark_suffix_with_optional_U_vowel()

    def __r_mark_sU(self):
        if not self.__r_check_vowel_harmony():
            return False
        if not self.in_grouping_b(TurkishStemmer.g_U):
            return False
        return self.__r_mark_suffix_with_optional_s_consonant()

    def __r_mark_lArI(self):
        return self.find_among_b(TurkishStemmer.a_1) != 0

    def __r_mark_yU(self):
        if not self.__r_check_vowel_harmony():
            return False
        if not self.in_grouping_b(TurkishStemmer.g_U):
            return False
        return self.__r_mark_suffix_with_optional_y_consonant()

    def __r_mark_nU(self):
        if not self.__r_check_vowel_harmony():
            return False
        return self.find_among_b(TurkishStemmer.a_2) != 0

    def __r_mark_nUn(self):
        if not self.__r_check_vowel_harmony():
            return False
        if self.find_among_b(TurkishStemmer.a_3) == 0:
            return False
        return self.__r_mark_suffix_with_optional_n_consonant()

    def __r_mark_yA(self):
        if not self.__r_check_vowel_harmony():
            return False
        if self.find_among_b(TurkishStemmer.a_4) == 0:
            return False
        return self.__r_mark_suffix_with_optional_y_consonant()

    def __r_mark_nA(self):
        if not self.__r_check_vowel_harmony():
            return False
        return self.find_among_b(TurkishStemmer.a_5) != 0

    def __r_mark_DA(self):
        if not self.__r_check_vowel_harmony():
            return False
        return self.find_among_b(TurkishStemmer.a_6) != 0

    def __r_mark_ndA(self):
        if not self.__r_check_vowel_harmony():
            return False
        return self.find_among_b(TurkishStemmer.a_7) != 0

    def __r_mark_DAn(self):
        if not self.__r_check_vowel_harmony():
            return False
        return self.find_among_b(TurkishStemmer.a_8) != 0

    def __r_mark_ndAn(self):
        if not self.__r_check_vowel_harmony():
            return False
        return self.find_among_b(TurkishStemmer.a_9) != 0

    def __r_mark_ylA(self):
        if not self.__r_check_vowel_harmony():
            return False
        if self.find_among_b(TurkishStemmer.a_10) == 0:
            return False
        return self.__r_mark_suffix_with_optional_y_consonant()

    def __r_mark_ncA(self):
        if not self.__r_check_vowel_harmony():
            return False
        if self.find_among_b(TurkishStemmer.a_11) == 0:
            return False
        return self.__r_mark_suffix_with_optional_n_consonant()

    def __r_mark_yUm(self):
        if not self.__r_check_vowel_harmony():
            return False
        if self.find_among_b(TurkishStemmer.a_12) == 0:
            return False
        return self.__r_mark_suffix_with_optional_y_consonant()

    def __r_mark_sUn(self):
        if not self.__r_check_vowel_harmony():
            return False
        return self.find_among_b(TurkishStemmer.a_13) != 0

    def __r_mark_yUz(self):
        if not self.__r_check_vowel_harmony():
            return False
        if self.find_among_b(TurkishStemmer.a_14) == 0:
            return False
        return self.__r_mark_suffix_with_optional_y_consonant()

    def __r_mark_sUnUz(self):
        return self.find_among_b(TurkishStemmer.a_15) != 0

    def __r_mark_lAr(self):
        if not self.__r_check_vowel_harmony():
            return False
        return self.find_among_b(TurkishStemmer.a_16) != 0

    def __r_mark_nUz(self):
        if not self.__r_check_vowel_harmony():
            return False
        return self.find_among_b(TurkishStemmer.a_17) != 0

    def __r_mark_DUr(self):
        if not self.__r_check_vowel_harmony():
            return False
        return self.find_among_b(TurkishStemmer.a_18) != 0

    def __r_mark_cAsInA(self):
        return self.find_among_b(TurkishStemmer.a_19) != 0

    def __r_mark_yDU(self):
        if not self.__r_check_vowel_harmony():
            return False
        if self.find_among_b(TurkishStemmer.a_20) == 0:
            return False
        return self.__r_mark_suffix_with_optional_y_consonant()

    def __r_mark_ysA(self):
        if self.find_among_b(TurkishStemmer.a_21) == 0:
            return False
        return self.__r_mark_suffix_with_optional_y_consonant()

    def __r_mark_ymUs_(self):
        if not self.__r_check_vowel_harmony():
            return False
        if self.find_among_b(TurkishStemmer.a_22) == 0:
            return False
        return self.__r_mark_suffix_with_optional_y_consonant()

    def __r_mark_yken(self):
        if not self.eq_s_b("ken"):
            return False
        return self.__r_mark_suffix_with_optional_y_consonant()

    def __r_stem_nominal_verb_suffixes(self):
        self.ket = self.cursor
        self.B_continue_stemming_noun_suffixes = True
        while True:
            v_1 = self.limit - self.cursor
            try:
                while True:
                    v_2 = self.limit - self.cursor
                    try:
                        if not self.__r_mark_ymUs_():
                            raise lab1()
                        break
                    except lab1: pass
                    self.cursor = self.limit - v_2
                    try:
                        if not self.__r_mark_yDU():
                            raise lab1()
                        break
                    except lab1: pass
                    self.cursor = self.limit - v_2
                    try:
                        if not self.__r_mark_ysA():
                            raise lab1()
                        break
                    except lab1: pass
                    self.cursor = self.limit - v_2
                    if not self.__r_mark_yken():
                        raise lab0()
                    break
                break
            except lab0: pass
            self.cursor = self.limit - v_1
            try:
                if not self.__r_mark_cAsInA():
                    raise lab0()
                while True:
                    v_3 = self.limit - self.cursor
                    try:
                        if not self.__r_mark_sUnUz():
                            raise lab1()
                        break
                    except lab1: pass
                    self.cursor = self.limit - v_3
                    try:
                        if not self.__r_mark_lAr():
                            raise lab1()
                        break
                    except lab1: pass
                    self.cursor = self.limit - v_3
                    try:
                        if not self.__r_mark_yUm():
                            raise lab1()
                        break
                    except lab1: pass
                    self.cursor = self.limit - v_3
                    try:
                        if not self.__r_mark_sUn():
                            raise lab1()
                        break
                    except lab1: pass
                    self.cursor = self.limit - v_3
                    try:
                        if not self.__r_mark_yUz():
                            raise lab1()
                        break
                    except lab1: pass
                    self.cursor = self.limit - v_3
                    break
                if not self.__r_mark_ymUs_():
                    raise lab0()
                break
            except lab0: pass
            self.cursor = self.limit - v_1
            try:
                if not self.__r_mark_lAr():
                    raise lab0()
                self.bra = self.cursor
                self.slice_del()
                v_4 = self.limit - self.cursor
                try:
                    self.ket = self.cursor
                    while True:
                        v_5 = self.limit - self.cursor
                        try:
                            if not self.__r_mark_DUr():
                                raise lab2()
                            break
                        except lab2: pass
                        self.cursor = self.limit - v_5
                        try:
                            if not self.__r_mark_yDU():
                                raise lab2()
                            break
                        except lab2: pass
                        self.cursor = self.limit - v_5
                        try:
                            if not self.__r_mark_ysA():
                                raise lab2()
                            break
                        except lab2: pass
                        self.cursor = self.limit - v_5
                        if not self.__r_mark_ymUs_():
                            self.cursor = self.limit - v_4
                            raise lab1()
                        break
                except lab1: pass
                self.B_continue_stemming_noun_suffixes = False
                break
            except lab0: pass
            self.cursor = self.limit - v_1
            try:
                if not self.__r_mark_nUz():
                    raise lab0()
                while True:
                    v_6 = self.limit - self.cursor
                    try:
                        if not self.__r_mark_yDU():
                            raise lab1()
                        break
                    except lab1: pass
                    self.cursor = self.limit - v_6
                    if not self.__r_mark_ysA():
                        raise lab0()
                    break
                break
            except lab0: pass
            self.cursor = self.limit - v_1
            try:
                while True:
                    v_7 = self.limit - self.cursor
                    try:
                        if not self.__r_mark_sUnUz():
                            raise lab1()
                        break
                    except lab1: pass
                    self.cursor = self.limit - v_7
                    try:
                        if not self.__r_mark_yUz():
                            raise lab1()
                        break
                    except lab1: pass
                    self.cursor = self.limit - v_7
                    try:
                        if not self.__r_mark_sUn():
                            raise lab1()
                        break
                    except lab1: pass
                    self.cursor = self.limit - v_7
                    if not self.__r_mark_yUm():
                        raise lab0()
                    break
                self.bra = self.cursor
                self.slice_del()
                v_8 = self.limit - self.cursor
                try:
                    self.ket = self.cursor
                    if not self.__r_mark_ymUs_():
                        self.cursor = self.limit - v_8
                        raise lab1()
                except lab1: pass
                break
            except lab0: pass
            self.cursor = self.limit - v_1
            if not self.__r_mark_DUr():
                return False
            self.bra = self.cursor
            self.slice_del()
            v_9 = self.limit - self.cursor
            try:
                self.ket = self.cursor
                while True:
                    v_10 = self.limit - self.cursor
                    try:
                        if not self.__r_mark_sUnUz():
                            raise lab1()
                        break
                    except lab1: pass
                    self.cursor = self.limit - v_10
                    try:
                        if not self.__r_mark_lAr():
                            raise lab1()
                        break
                    except lab1: pass
                    self.cursor = self.limit - v_10
                    try:
                        if not self.__r_mark_yUm():
                            raise lab1()
                        break
                    except lab1: pass
                    self.cursor = self.limit - v_10
                    try:
                        if not self.__r_mark_sUn():
                            raise lab1()
                        break
                    except lab1: pass
                    self.cursor = self.limit - v_10
                    try:
                        if not self.__r_mark_yUz():
                            raise lab1()
                        break
                    except lab1: pass
                    self.cursor = self.limit - v_10
                    break
                if not self.__r_mark_ymUs_():
                    self.cursor = self.limit - v_9
                    raise lab0()
            except lab0: pass
            break
        self.bra = self.cursor
        self.slice_del()
        return True

    def __r_stem_suffix_chain_before_ki(self):
        self.ket = self.cursor
        if not self.eq_s_b("ki"):
            return False
        while True:
            v_1 = self.limit - self.cursor
            try:
                if not self.__r_mark_DA():
                    raise lab0()
                self.bra = self.cursor
                self.slice_del()
                v_2 = self.limit - self.cursor
                try:
                    self.ket = self.cursor
                    while True:
                        v_3 = self.limit - self.cursor
                        try:
                            if not self.__r_mark_lAr():
                                raise lab2()
                            self.bra = self.cursor
                            self.slice_del()
                            v_4 = self.limit - self.cursor
                            try:
                                if not self.__r_stem_suffix_chain_before_ki():
                                    self.cursor = self.limit - v_4
                                    raise lab3()
                            except lab3: pass
                            break
                        except lab2: pass
                        self.cursor = self.limit - v_3
                        if not self.__r_mark_possessives():
                            self.cursor = self.limit - v_2
                            raise lab1()
                        self.bra = self.cursor
                        self.slice_del()
                        v_5 = self.limit - self.cursor
                        try:
                            self.ket = self.cursor
                            if not self.__r_mark_lAr():
                                self.cursor = self.limit - v_5
                                raise lab2()
                            self.bra = self.cursor
                            self.slice_del()
                            if not self.__r_stem_suffix_chain_before_ki():
                                self.cursor = self.limit - v_5
                                raise lab2()
                        except lab2: pass
                        break
                except lab1: pass
                break
            except lab0: pass
            self.cursor = self.limit - v_1
            try:
                if not self.__r_mark_nUn():
                    raise lab0()
                self.bra = self.cursor
                self.slice_del()
                v_6 = self.limit - self.cursor
                try:
                    self.ket = self.cursor
                    while True:
                        v_7 = self.limit - self.cursor
                        try:
                            if not self.__r_mark_lArI():
                                raise lab2()
                            self.bra = self.cursor
                            self.slice_del()
                            break
                        except lab2: pass
                        self.cursor = self.limit - v_7
                        try:
                            self.ket = self.cursor
                            while True:
                                v_8 = self.limit - self.cursor
                                try:
                                    if not self.__r_mark_possessives():
                                        raise lab3()
                                    break
                                except lab3: pass
                                self.cursor = self.limit - v_8
                                if not self.__r_mark_sU():
                                    raise lab2()
                                break
                            self.bra = self.cursor
                            self.slice_del()
                            v_9 = self.limit - self.cursor
                            try:
                                self.ket = self.cursor
                                if not self.__r_mark_lAr():
                                    self.cursor = self.limit - v_9
                                    raise lab3()
                                self.bra = self.cursor
                                self.slice_del()
                                if not self.__r_stem_suffix_chain_before_ki():
                                    self.cursor = self.limit - v_9
                                    raise lab3()
                            except lab3: pass
                            break
                        except lab2: pass
                        self.cursor = self.limit - v_7
                        if not self.__r_stem_suffix_chain_before_ki():
                            self.cursor = self.limit - v_6
                            raise lab1()
                        break
                except lab1: pass
                break
            except lab0: pass
            self.cursor = self.limit - v_1
            if not self.__r_mark_ndA():
                return False
            while True:
                v_10 = self.limit - self.cursor
                try:
                    if not self.__r_mark_lArI():
                        raise lab0()
                    self.bra = self.cursor
                    self.slice_del()
                    break
                except lab0: pass
                self.cursor = self.limit - v_10
                try:
                    if not self.__r_mark_sU():
                        raise lab0()
                    self.bra = self.cursor
                    self.slice_del()
                    v_11 = self.limit - self.cursor
                    try:
                        self.ket = self.cursor
                        if not self.__r_mark_lAr():
                            self.cursor = self.limit - v_11
                            raise lab1()
                        self.bra = self.cursor
                        self.slice_del()
                        if not self.__r_stem_suffix_chain_before_ki():
                            self.cursor = self.limit - v_11
                            raise lab1()
                    except lab1: pass
                    break
                except lab0: pass
                self.cursor = self.limit - v_10
                if not self.__r_stem_suffix_chain_before_ki():
                    return False
                break
            break
        return True

    def __r_stem_noun_suffixes(self):
        while True:
            v_1 = self.limit - self.cursor
            try:
                self.ket = self.cursor
                if not self.__r_mark_lAr():
                    raise lab0()
                self.bra = self.cursor
                self.slice_del()
                v_2 = self.limit - self.cursor
                try:
                    if not self.__r_stem_suffix_chain_before_ki():
                        self.cursor = self.limit - v_2
                        raise lab1()
                except lab1: pass
                break
            except lab0: pass
            self.cursor = self.limit - v_1
            try:
                self.ket = self.cursor
                if not self.__r_mark_ncA():
                    raise lab0()
                self.bra = self.cursor
                self.slice_del()
                v_3 = self.limit - self.cursor
                try:
                    while True:
                        v_4 = self.limit - self.cursor
                        try:
                            self.ket = self.cursor
                            if not self.__r_mark_lArI():
                                raise lab2()
                            self.bra = self.cursor
                            self.slice_del()
                            break
                        except lab2: pass
                        self.cursor = self.limit - v_4
                        try:
                            self.ket = self.cursor
                            while True:
                                v_5 = self.limit - self.cursor
                                try:
                                    if not self.__r_mark_possessives():
                                        raise lab3()
                                    break
                                except lab3: pass
                                self.cursor = self.limit - v_5
                                if not self.__r_mark_sU():
                                    raise lab2()
                                break
                            self.bra = self.cursor
                            self.slice_del()
                            v_6 = self.limit - self.cursor
                            try:
                                self.ket = self.cursor
                                if not self.__r_mark_lAr():
                                    self.cursor = self.limit - v_6
                                    raise lab3()
                                self.bra = self.cursor
                                self.slice_del()
                                if not self.__r_stem_suffix_chain_before_ki():
                                    self.cursor = self.limit - v_6
                                    raise lab3()
                            except lab3: pass
                            break
                        except lab2: pass
                        self.cursor = self.limit - v_4
                        self.ket = self.cursor
                        if not self.__r_mark_lAr():
                            self.cursor = self.limit - v_3
                            raise lab1()
                        self.bra = self.cursor
                        self.slice_del()
                        if not self.__r_stem_suffix_chain_before_ki():
                            self.cursor = self.limit - v_3
                            raise lab1()
                        break
                except lab1: pass
                break
            except lab0: pass
            self.cursor = self.limit - v_1
            try:
                self.ket = self.cursor
                while True:
                    v_7 = self.limit - self.cursor
                    try:
                        if not self.__r_mark_ndA():
                            raise lab1()
                        break
                    except lab1: pass
                    self.cursor = self.limit - v_7
                    if not self.__r_mark_nA():
                        raise lab0()
                    break
                while True:
                    v_8 = self.limit - self.cursor
                    try:
                        if not self.__r_mark_lArI():
                            raise lab1()
                        self.bra = self.cursor
                        self.slice_del()
                        break
                    except lab1: pass
                    self.cursor = self.limit - v_8
                    try:
                        if not self.__r_mark_sU():
                            raise lab1()
                        self.bra = self.cursor
                        self.slice_del()
                        v_9 = self.limit - self.cursor
                        try:
                            self.ket = self.cursor
                            if not self.__r_mark_lAr():
                                self.cursor = self.limit - v_9
                                raise lab2()
                            self.bra = self.cursor
                            self.slice_del()
                            if not self.__r_stem_suffix_chain_before_ki():
                                self.cursor = self.limit - v_9
                                raise lab2()
                        except lab2: pass
                        break
                    except lab1: pass
                    self.cursor = self.limit - v_8
                    if not self.__r_stem_suffix_chain_before_ki():
                        raise lab0()
                    break
                break
            except lab0: pass
            self.cursor = self.limit - v_1
            try:
                self.ket = self.cursor
                while True:
                    v_10 = self.limit - self.cursor
                    try:
                        if not self.__r_mark_ndAn():
                            raise lab1()
                        break
                    except lab1: pass
                    self.cursor = self.limit - v_10
                    if not self.__r_mark_nU():
                        raise lab0()
                    break
                while True:
                    v_11 = self.limit - self.cursor
                    try:
                        if not self.__r_mark_sU():
                            raise lab1()
                        self.bra = self.cursor
                        self.slice_del()
                        v_12 = self.limit - self.cursor
                        try:
                            self.ket = self.cursor
                            if not self.__r_mark_lAr():
                                self.cursor = self.limit - v_12
                                raise lab2()
                            self.bra = self.cursor
                            self.slice_del()
                            if not self.__r_stem_suffix_chain_before_ki():
                                self.cursor = self.limit - v_12
                                raise lab2()
                        except lab2: pass
                        break
                    except lab1: pass
                    self.cursor = self.limit - v_11
                    if not self.__r_mark_lArI():
                        raise lab0()
                    break
                break
            except lab0: pass
            self.cursor = self.limit - v_1
            try:
                self.ket = self.cursor
                if not self.__r_mark_DAn():
                    raise lab0()
                self.bra = self.cursor
                self.slice_del()
                v_13 = self.limit - self.cursor
                try:
                    self.ket = self.cursor
                    while True:
                        v_14 = self.limit - self.cursor
                        try:
                            if not self.__r_mark_possessives():
                                raise lab2()
                            self.bra = self.cursor
                            self.slice_del()
                            v_15 = self.limit - self.cursor
                            try:
                                self.ket = self.cursor
                                if not self.__r_mark_lAr():
                                    self.cursor = self.limit - v_15
                                    raise lab3()
                                self.bra = self.cursor
                                self.slice_del()
                                if not self.__r_stem_suffix_chain_before_ki():
                                    self.cursor = self.limit - v_15
                                    raise lab3()
                            except lab3: pass
                            break
                        except lab2: pass
                        self.cursor = self.limit - v_14
                        try:
                            if not self.__r_mark_lAr():
                                raise lab2()
                            self.bra = self.cursor
                            self.slice_del()
                            v_16 = self.limit - self.cursor
                            try:
                                if not self.__r_stem_suffix_chain_before_ki():
                                    self.cursor = self.limit - v_16
                                    raise lab3()
                            except lab3: pass
                            break
                        except lab2: pass
                        self.cursor = self.limit - v_14
                        if not self.__r_stem_suffix_chain_before_ki():
                            self.cursor = self.limit - v_13
                            raise lab1()
                        break
                except lab1: pass
                break
            except lab0: pass
            self.cursor = self.limit - v_1
            try:
                self.ket = self.cursor
                while True:
                    v_17 = self.limit - self.cursor
                    try:
                        if not self.__r_mark_nUn():
                            raise lab1()
                        break
                    except lab1: pass
                    self.cursor = self.limit - v_17
                    if not self.__r_mark_ylA():
                        raise lab0()
                    break
                self.bra = self.cursor
                self.slice_del()
                v_18 = self.limit - self.cursor
                try:
                    while True:
                        v_19 = self.limit - self.cursor
                        try:
                            self.ket = self.cursor
                            if not self.__r_mark_lAr():
                                raise lab2()
                            self.bra = self.cursor
                            self.slice_del()
                            if not self.__r_stem_suffix_chain_before_ki():
                                raise lab2()
                            break
                        except lab2: pass
                        self.cursor = self.limit - v_19
                        try:
                            self.ket = self.cursor
                            while True:
                                v_20 = self.limit - self.cursor
                                try:
                                    if not self.__r_mark_possessives():
                                        raise lab3()
                                    break
                                except lab3: pass
                                self.cursor = self.limit - v_20
                                if not self.__r_mark_sU():
                                    raise lab2()
                                break
                            self.bra = self.cursor
                            self.slice_del()
                            v_21 = self.limit - self.cursor
                            try:
                                self.ket = self.cursor
                                if not self.__r_mark_lAr():
                                    self.cursor = self.limit - v_21
                                    raise lab3()
                                self.bra = self.cursor
                                self.slice_del()
                                if not self.__r_stem_suffix_chain_before_ki():
                                    self.cursor = self.limit - v_21
                                    raise lab3()
                            except lab3: pass
                            break
                        except lab2: pass
                        self.cursor = self.limit - v_19
                        if not self.__r_stem_suffix_chain_before_ki():
                            self.cursor = self.limit - v_18
                            raise lab1()
                        break
                except lab1: pass
                break
            except lab0: pass
            self.cursor = self.limit - v_1
            try:
                self.ket = self.cursor
                if not self.__r_mark_lArI():
                    raise lab0()
                self.bra = self.cursor
                self.slice_del()
                break
            except lab0: pass
            self.cursor = self.limit - v_1
            try:
                if not self.__r_stem_suffix_chain_before_ki():
                    raise lab0()
                break
            except lab0: pass
            self.cursor = self.limit - v_1
            try:
                self.ket = self.cursor
                while True:
                    v_22 = self.limit - self.cursor
                    try:
                        if not self.__r_mark_DA():
                            raise lab1()
                        break
                    except lab1: pass
                    self.cursor = self.limit - v_22
                    try:
                        if not self.__r_mark_yU():
                            raise lab1()
                        break
                    except lab1: pass
                    self.cursor = self.limit - v_22
                    if not self.__r_mark_yA():
                        raise lab0()
                    break
                self.bra = self.cursor
                self.slice_del()
                v_23 = self.limit - self.cursor
                try:
                    self.ket = self.cursor
                    while True:
                        v_24 = self.limit - self.cursor
                        try:
                            if not self.__r_mark_possessives():
                                raise lab2()
                            self.bra = self.cursor
                            self.slice_del()
                            v_25 = self.limit - self.cursor
                            try:
                                self.ket = self.cursor
                                if not self.__r_mark_lAr():
                                    self.cursor = self.limit - v_25
                                    raise lab3()
                            except lab3: pass
                            break
                        except lab2: pass
                        self.cursor = self.limit - v_24
                        if not self.__r_mark_lAr():
                            self.cursor = self.limit - v_23
                            raise lab1()
                        break
                    self.bra = self.cursor
                    self.slice_del()
                    self.ket = self.cursor
                    if not self.__r_stem_suffix_chain_before_ki():
                        self.cursor = self.limit - v_23
                        raise lab1()
                except lab1: pass
                break
            except lab0: pass
            self.cursor = self.limit - v_1
            self.ket = self.cursor
            while True:
                v_26 = self.limit - self.cursor
                try:
                    if not self.__r_mark_possessives():
                        raise lab0()
                    break
                except lab0: pass
                self.cursor = self.limit - v_26
                if not self.__r_mark_sU():
                    return False
                break
            self.bra = self.cursor
            self.slice_del()
            v_27 = self.limit - self.cursor
            try:
                self.ket = self.cursor
                if not self.__r_mark_lAr():
                    self.cursor = self.limit - v_27
                    raise lab0()
                self.bra = self.cursor
                self.slice_del()
                if not self.__r_stem_suffix_chain_before_ki():
                    self.cursor = self.limit - v_27
                    raise lab0()
            except lab0: pass
            break
        return True

    def __r_post_process_last_consonants(self):
        self.ket = self.cursor
        among_var = self.find_among_b(TurkishStemmer.a_23)
        if among_var == 0:
            return False
        self.bra = self.cursor
        self.slice_from(TurkishStemmer.as_23[among_var - 1])
        return True

    def __r_append_U_to_stems_ending_with_d_or_g(self):
        self.ket = self.cursor
        self.bra = self.cursor
        while True:
            try:
                if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "d":
                    raise lab0()
                self.cursor -= 1
                break
            except lab0: pass
            if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "g":
                return False
            self.cursor -= 1
            break
        if not self.go_out_grouping_b(TurkishStemmer.g_vowel):
            return False
        while True:
            v_1 = self.limit - self.cursor
            try:
                while True:
                    try:
                        if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "a":
                            raise lab1()
                        self.cursor -= 1
                        break
                    except lab1: pass
                    if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "ı":
                        raise lab0()
                    self.cursor -= 1
                    break
                self.slice_from("ı")
                break
            except lab0: pass
            self.cursor = self.limit - v_1
            try:
                while True:
                    try:
                        if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "e":
                            raise lab1()
                        self.cursor -= 1
                        break
                    except lab1: pass
                    if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "i":
                        raise lab0()
                    self.cursor -= 1
                    break
                self.slice_from("i")
                break
            except lab0: pass
            self.cursor = self.limit - v_1
            try:
                while True:
                    try:
                        if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "o":
                            raise lab1()
                        self.cursor -= 1
                        break
                    except lab1: pass
                    if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "u":
                        raise lab0()
                    self.cursor -= 1
                    break
                self.slice_from("u")
                break
            except lab0: pass
            self.cursor = self.limit - v_1
            while True:
                try:
                    if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "ö":
                        raise lab0()
                    self.cursor -= 1
                    break
                except lab0: pass
                if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "ü":
                    return False
                self.cursor -= 1
                break
            self.slice_from("ü")
            break
        return True

    def __r_is_reserved_word(self):
        if not self.eq_s_b("ad"):
            return False
        v_1 = self.limit - self.cursor
        try:
            if not self.eq_s_b("soy"):
                self.cursor = self.limit - v_1
                raise lab0()
        except lab0: pass
        return self.cursor <= self.limit_backward

    def __r_remove_proper_noun_suffix(self):
        v_1 = self.cursor
        try:
            self.bra = self.cursor
            while True:
                v_2 = self.cursor
                try:
                    try:
                        if self.cursor == self.limit or self.current[self.cursor] != "'":
                            raise lab2()
                        self.cursor += 1
                        raise lab1()
                    except lab2: pass
                    self.cursor = v_2
                    break
                except lab1: pass
                self.cursor = v_2
                if self.cursor >= self.limit:
                    raise lab0()
                self.cursor += 1
            self.ket = self.cursor
            self.slice_del()
        except lab0: pass
        self.cursor = v_1
        v_3 = self.cursor
        try:
            if self.cursor + 2 > self.limit:
                raise lab0()
            self.cursor += 2
            while True:
                v_4 = self.cursor
                try:
                    if self.cursor == self.limit or self.current[self.cursor] != "'":
                        raise lab1()
                    self.cursor += 1
                    self.cursor = v_4
                    break
                except lab1: pass
                self.cursor = v_4
                if self.cursor >= self.limit:
                    raise lab0()
                self.cursor += 1
            self.bra = self.cursor
            self.cursor = self.limit
            self.ket = self.cursor
            self.slice_del()
        except lab0: pass
        self.cursor = v_3
        return True

    def __r_more_than_one_syllable_word(self):
        v_1 = self.cursor
        for _ in 0, 0:
            if not self.go_out_grouping(TurkishStemmer.g_vowel):
                return False
            self.cursor += 1
        self.cursor = v_1
        return True

    def __r_postlude(self):
        self.limit_backward = self.cursor
        self.cursor = self.limit
        v_1 = self.limit - self.cursor
        try:
            if not self.__r_is_reserved_word():
                raise lab0()
            return False
        except lab0: pass
        self.cursor = self.limit - v_1
        v_2 = self.limit - self.cursor
        self.__r_append_U_to_stems_ending_with_d_or_g()
        self.cursor = self.limit - v_2
        v_3 = self.limit - self.cursor
        self.__r_post_process_last_consonants()
        self.cursor = self.limit - v_3
        self.cursor = self.limit_backward
        return True

    def _stem(self):
        self.__r_remove_proper_noun_suffix()
        if not self.__r_more_than_one_syllable_word():
            return False
        self.limit_backward = self.cursor
        self.cursor = self.limit
        v_1 = self.limit - self.cursor
        self.__r_stem_nominal_verb_suffixes()
        self.cursor = self.limit - v_1
        if not self.B_continue_stemming_noun_suffixes:
            return False
        v_2 = self.limit - self.cursor
        self.__r_stem_noun_suffixes()
        self.cursor = self.limit - v_2
        self.cursor = self.limit_backward
        return self.__r_postlude()

    a_0 = [
        Among("m", -1, -1),
        Among("n", -1, -1),
        Among("miz", -1, -1),
        Among("niz", -1, -1),
        Among("muz", -1, -1),
        Among("nuz", -1, -1),
        Among("müz", -1, -1),
        Among("nüz", -1, -1),
        Among("mız", -1, -1),
        Among("nız", -1, -1)
    ]

    a_1 = [
        Among("leri", -1, -1),
        Among("ları", -1, -1)
    ]

    a_2 = [
        Among("ni", -1, -1),
        Among("nu", -1, -1),
        Among("nü", -1, -1),
        Among("nı", -1, -1)
    ]

    a_3 = [
        Among("in", -1, -1),
        Among("un", -1, -1),
        Among("ün", -1, -1),
        Among("ın", -1, -1)
    ]

    a_4 = [
        Among("a", -1, -1),
        Among("e", -1, -1)
    ]

    a_5 = [
        Among("na", -1, -1),
        Among("ne", -1, -1)
    ]

    a_6 = [
        Among("da", -1, -1),
        Among("ta", -1, -1),
        Among("de", -1, -1),
        Among("te", -1, -1)
    ]

    a_7 = [
        Among("nda", -1, -1),
        Among("nde", -1, -1)
    ]

    a_8 = [
        Among("dan", -1, -1),
        Among("tan", -1, -1),
        Among("den", -1, -1),
        Among("ten", -1, -1)
    ]

    a_9 = [
        Among("ndan", -1, -1),
        Among("nden", -1, -1)
    ]

    a_10 = [
        Among("la", -1, -1),
        Among("le", -1, -1)
    ]

    a_11 = [
        Among("ca", -1, -1),
        Among("ce", -1, -1)
    ]

    a_12 = [
        Among("im", -1, -1),
        Among("um", -1, -1),
        Among("üm", -1, -1),
        Among("ım", -1, -1)
    ]

    a_13 = [
        Among("sin", -1, -1),
        Among("sun", -1, -1),
        Among("sün", -1, -1),
        Among("sın", -1, -1)
    ]

    a_14 = [
        Among("iz", -1, -1),
        Among("uz", -1, -1),
        Among("üz", -1, -1),
        Among("ız", -1, -1)
    ]

    a_15 = [
        Among("siniz", -1, -1),
        Among("sunuz", -1, -1),
        Among("sünüz", -1, -1),
        Among("sınız", -1, -1)
    ]

    a_16 = [
        Among("lar", -1, -1),
        Among("ler", -1, -1)
    ]

    a_17 = [
        Among("niz", -1, -1),
        Among("nuz", -1, -1),
        Among("nüz", -1, -1),
        Among("nız", -1, -1)
    ]

    a_18 = [
        Among("dir", -1, -1),
        Among("tir", -1, -1),
        Among("dur", -1, -1),
        Among("tur", -1, -1),
        Among("dür", -1, -1),
        Among("tür", -1, -1),
        Among("dır", -1, -1),
        Among("tır", -1, -1)
    ]

    a_19 = [
        Among("casına", -1, -1),
        Among("cesine", -1, -1)
    ]

    a_20 = [
        Among("di", -1, -1),
        Among("ti", -1, -1),
        Among("dik", -1, -1),
        Among("tik", -1, -1),
        Among("duk", -1, -1),
        Among("tuk", -1, -1),
        Among("dük", -1, -1),
        Among("tük", -1, -1),
        Among("dık", -1, -1),
        Among("tık", -1, -1),
        Among("dim", -1, -1),
        Among("tim", -1, -1),
        Among("dum", -1, -1),
        Among("tum", -1, -1),
        Among("düm", -1, -1),
        Among("tüm", -1, -1),
        Among("dım", -1, -1),
        Among("tım", -1, -1),
        Among("din", -1, -1),
        Among("tin", -1, -1),
        Among("dun", -1, -1),
        Among("tun", -1, -1),
        Among("dün", -1, -1),
        Among("tün", -1, -1),
        Among("dın", -1, -1),
        Among("tın", -1, -1),
        Among("du", -1, -1),
        Among("tu", -1, -1),
        Among("dü", -1, -1),
        Among("tü", -1, -1),
        Among("dı", -1, -1),
        Among("tı", -1, -1)
    ]

    a_21 = [
        Among("sa", -1, -1),
        Among("se", -1, -1),
        Among("sak", -1, -1),
        Among("sek", -1, -1),
        Among("sam", -1, -1),
        Among("sem", -1, -1),
        Among("san", -1, -1),
        Among("sen", -1, -1)
    ]

    a_22 = [
        Among("miş", -1, -1),
        Among("muş", -1, -1),
        Among("müş", -1, -1),
        Among("mış", -1, -1)
    ]

    a_23 = [
        Among("b", -1, 1),
        Among("c", -1, 2),
        Among("d", -1, 3),
        Among("ğ", -1, 4)
    ]
    as_23 = ("p", "ç", "t", "k")


class lab0(BaseException): pass


class lab1(BaseException): pass


class lab2(BaseException): pass


class lab3(BaseException): pass



# Generated from swedish.sbl by Snowball 3.1.1 - https://snowballstem.org/

from .basestemmer import BaseStemmer
from .among import Among


class SwedishStemmer(BaseStemmer):
    '''
    This class implements the stemming algorithm defined by a snowball script.
    Generated from swedish.sbl by Snowball 3.1.1 - https://snowballstem.org/
    '''

    g_v = {"a", "e", "i", "o", "u", "y", "ä", "å", "ö"}

    g_s_ending = {"b", "c", "d", "f", "g", "h", "j", "k", "l", "m", "n", "o", "p", "r", "t", "v", "y"}

    g_ost_ending = {"i", "k", "l", "n", "p", "r", "t", "u", "v"}

    I_p1 = 0

    def __r_mark_regions(self):
        self.I_p1 = self.limit
        v_1 = self.cursor
        if self.cursor + 3 > self.limit:
            return False
        self.cursor += 3
        I_x = self.cursor
        self.cursor = v_1
        if not self.go_out_grouping(SwedishStemmer.g_v):
            return False
        self.cursor += 1
        if not self.go_in_grouping(SwedishStemmer.g_v):
            return False
        self.cursor += 1
        self.I_p1 = self.cursor
        try:
            if self.I_p1 >= I_x:
                raise lab0()
            self.I_p1 = I_x
        except lab0: pass
        return True

    def __r_et_condition(self):
        v_1 = self.limit - self.cursor
        if not self.out_grouping_b(SwedishStemmer.g_v):
            return False
        if not self.in_grouping_b(SwedishStemmer.g_v):
            return False
        if self.cursor <= self.limit_backward:
            return False
        self.cursor = self.limit - v_1
        v_2 = self.limit - self.cursor
        try:
            if self.find_among_b(SwedishStemmer.a_0) == 0:
                raise lab0()
            return False
        except lab0: pass
        self.cursor = self.limit - v_2
        return True

    def __r_main_suffix(self):
        if self.cursor < self.I_p1:
            return False
        v_2 = self.limit_backward
        self.limit_backward = self.I_p1
        self.ket = self.cursor
        among_var = self.find_among_b(SwedishStemmer.a_1)
        if among_var == 0:
            self.limit_backward = v_2
            return False
        self.bra = self.cursor
        self.limit_backward = v_2
        if among_var == 1:
            self.slice_del()
        elif among_var == 2:
            while True:
                v_3 = self.limit - self.cursor
                try:
                    if not self.eq_s_b("et"):
                        raise lab0()
                    if not self.__r_et_condition():
                        raise lab0()
                    self.bra = self.cursor
                    break
                except lab0: pass
                self.cursor = self.limit - v_3
                if not self.in_grouping_b(SwedishStemmer.g_s_ending):
                    return False
                break
            self.slice_del()
        else:
            if not self.__r_et_condition():
                return False
            self.slice_del()
        return True

    def __r_consonant_pair(self):
        if self.cursor < self.I_p1:
            return False
        v_2 = self.limit_backward
        self.limit_backward = self.I_p1
        v_3 = self.limit - self.cursor
        if self.find_among_b(SwedishStemmer.a_2) == 0:
            self.limit_backward = v_2
            return False
        self.cursor = self.limit - v_3
        self.ket = self.cursor
        if self.cursor <= self.limit_backward:
            self.limit_backward = v_2
            return False
        self.cursor -= 1
        self.bra = self.cursor
        self.slice_del()
        self.limit_backward = v_2
        return True

    def __r_other_suffix(self):
        if self.cursor < self.I_p1:
            return False
        v_2 = self.limit_backward
        self.limit_backward = self.I_p1
        self.ket = self.cursor
        among_var = self.find_among_b(SwedishStemmer.a_3)
        if among_var == 0:
            self.limit_backward = v_2
            return False
        self.bra = self.cursor
        self.limit_backward = v_2
        if among_var == 1:
            self.slice_del()
        elif among_var == 2:
            if not self.in_grouping_b(SwedishStemmer.g_ost_ending):
                return False
            self.slice_from("ös")
        else:
            self.slice_from("full")
        return True

    def _stem(self):
        v_1 = self.cursor
        self.__r_mark_regions()
        self.cursor = v_1
        self.limit_backward = self.cursor
        self.cursor = self.limit
        v_2 = self.limit - self.cursor
        self.__r_main_suffix()
        self.cursor = self.limit - v_2
        v_3 = self.limit - self.cursor
        self.__r_consonant_pair()
        self.cursor = self.limit - v_3
        v_4 = self.limit - self.cursor
        self.__r_other_suffix()
        self.cursor = self.limit - v_4
        self.cursor = self.limit_backward
        return True

    a_0 = [
        Among("fab", -1, -1),
        Among("h", -1, -1),
        Among("pak", -1, -1),
        Among("rak", -1, -1),
        Among("stak", -1, -1),
        Among("kom", -1, -1),
        Among("iet", -1, -1),
        Among("cit", -1, -1),
        Among("dit", -1, -1),
        Among("alit", -1, -1),
        Among("ilit", -1, -1),
        Among("mit", -1, -1),
        Among("nit", -1, -1),
        Among("pit", -1, -1),
        Among("rit", -1, -1),
        Among("sit", -1, -1),
        Among("tit", -1, -1),
        Among("uit", -1, -1),
        Among("ivit", -1, -1),
        Among("kvit", -1, -1),
        Among("xit", -1, -1)
    ]

    a_1 = [
        Among("a", -1, 1),
        Among("arna", 0, 1),
        Among("erna", 0, 1),
        Among("heterna", 2, 1),
        Among("orna", 0, 1),
        Among("ad", -1, 1),
        Among("e", -1, 1),
        Among("ade", 6, 1),
        Among("ande", 6, 1),
        Among("arne", 6, 1),
        Among("are", 6, 1),
        Among("aste", 6, 1),
        Among("en", -1, 1),
        Among("anden", 12, 1),
        Among("aren", 12, 1),
        Among("heten", 12, 1),
        Among("ern", -1, 1),
        Among("ar", -1, 1),
        Among("er", -1, 1),
        Among("heter", 18, 1),
        Among("or", -1, 1),
        Among("s", -1, 2),
        Among("as", 21, 1),
        Among("arnas", 22, 1),
        Among("ernas", 22, 1),
        Among("ornas", 22, 1),
        Among("es", 21, 1),
        Among("ades", 26, 1),
        Among("andes", 26, 1),
        Among("ens", 21, 1),
        Among("arens", 29, 1),
        Among("hetens", 29, 1),
        Among("erns", 21, 1),
        Among("at", -1, 1),
        Among("et", -1, 3),
        Among("andet", 34, 1),
        Among("het", 34, 1),
        Among("ast", -1, 1)
    ]

    a_2 = [
        Among("dd", -1, -1),
        Among("gd", -1, -1),
        Among("nn", -1, -1),
        Among("dt", -1, -1),
        Among("gt", -1, -1),
        Among("kt", -1, -1),
        Among("tt", -1, -1)
    ]

    a_3 = [
        Among("ig", -1, 1),
        Among("lig", 0, 1),
        Among("els", -1, 1),
        Among("fullt", -1, 3),
        Among("öst", -1, 2)
    ]


class lab0(BaseException): pass



# Generated from persian.sbl by Snowball 3.1.1 - https://snowballstem.org/

from .basestemmer import BaseStemmer
from .among import Among


class PersianStemmer(BaseStemmer):
    '''
    This class implements the stemming algorithm defined by a snowball script.
    Generated from persian.sbl by Snowball 3.1.1 - https://snowballstem.org/
    '''

    I_p1 = 0
    B_remove_verb_person_endings = False
    B_saw_present_prefix = False

    def __r_Normalize_Characters(self):
        while True:
            v_1 = self.cursor
            try:
                self.bra = self.cursor
                among_var = self.find_among(PersianStemmer.a_0)
                self.ket = self.cursor
                if among_var == 1:
                    self.slice_from("\u06A9")
                elif among_var == 2:
                    self.slice_from("\u06CC")
                elif among_var == 3:
                    self.slice_from("\u0647")
                elif among_var == 4:
                    self.slice_from("\u0627")
                elif among_var == 5:
                    self.slice_from("\u0648")
                elif among_var == 6:
                    self.slice_del()
                else:
                    if self.cursor >= self.limit:
                        raise lab0()
                    self.cursor += 1
                continue
            except lab0: pass
            self.cursor = v_1
            break
        return True

    def __r_Prefixes(self):
        self.bra = self.cursor
        among_var = self.find_among(PersianStemmer.a_1)
        if among_var == 0:
            return False
        self.ket = self.cursor
        if among_var == 1:
            if self.cursor + 2 > self.limit:
                return False
            self.cursor += 2
            self.B_saw_present_prefix = True
        else:
            if self.cursor + 2 > self.limit:
                return False
            self.cursor += 2
            self.slice_del()
            self.B_saw_present_prefix = True
        return True

    def __r_Delete_ZWNJ(self):
        while True:
            v_1 = self.cursor
            try:
                while True:
                    v_2 = self.cursor
                    try:
                        self.bra = self.cursor
                        if self.cursor == self.limit or self.current[self.cursor] != "\u200C":
                            raise lab1()
                        self.cursor += 1
                        self.ket = self.cursor
                        self.slice_del()
                        self.cursor = v_2
                        break
                    except lab1: pass
                    self.cursor = v_2
                    if self.cursor >= self.limit:
                        raise lab0()
                    self.cursor += 1
                continue
            except lab0: pass
            self.cursor = v_1
            break
        return True

    def __r_R1(self):
        return self.I_p1 <= self.cursor

    def __r_Protect_Lexical_AN(self):
        v_1 = self.limit - self.cursor
        try:
            if not self.__r_AN_Exception():
                raise lab0()
            return False
        except lab0: pass
        self.cursor = self.limit - v_1
        v_2 = self.limit - self.cursor
        try:
            if self.find_among_b(PersianStemmer.a_2) == 0:
                raise lab0()
            return False
        except lab0: pass
        self.cursor = self.limit - v_2
        return True

    def __r_AN_Exception(self):
        if self.find_among_b(PersianStemmer.a_3) == 0:
            return False
        if self.cursor > self.limit_backward:
            return False
        return True

    def __r_Irregular_Noun(self):
        self.ket = self.cursor
        among_var = self.find_among_b(PersianStemmer.a_4)
        if among_var == 0:
            return False
        self.bra = self.cursor
        self.slice_from(PersianStemmer.as_4[among_var - 1])
        return True

    def __r_Stem_Noun_or_Adjective(self):
        while True:
            v_1 = self.limit - self.cursor
            try:
                if not self.__r_Irregular_Noun():
                    raise lab0()
                break
            except lab0: pass
            self.cursor = self.limit - v_1
            if self.cursor < self.I_p1:
                return False
            v_3 = self.limit_backward
            self.limit_backward = self.I_p1
            self.ket = self.cursor
            among_var = self.find_among_b(PersianStemmer.a_5)
            if among_var == 0:
                self.limit_backward = v_3
                return False
            self.bra = self.cursor
            if among_var == 1:
                self.slice_del()
            else:
                if self.cursor <= self.limit_backward:
                    self.limit_backward = v_3
                    return False
                self.slice_del()
            self.limit_backward = v_3
            break
        return True

    def __r_Stem_Verb(self):
        while True:
            v_1 = self.limit - self.cursor
            try:
                self.ket = self.cursor
                if self.find_among_b(PersianStemmer.a_6) == 0:
                    raise lab0()
                self.bra = self.cursor
                if not self.__r_R1():
                    raise lab0()
                self.slice_del()
                break
            except lab0: pass
            self.cursor = self.limit - v_1
            self.ket = self.cursor
            among_var = self.find_among_b(PersianStemmer.a_7)
            if among_var == 0:
                return False
            self.bra = self.cursor
            if among_var == 1:
                if not self.B_remove_verb_person_endings:
                    return False
                if not self.__r_R1():
                    return False
                self.slice_del()
            elif among_var == 2:
                self.slice_from("\u0631\u0641\u062A")
            elif among_var == 3:
                if not self.__r_R1():
                    return False
                self.slice_del()
                self.B_remove_verb_person_endings = True
            elif among_var == 4:
                if self.cursor <= self.limit_backward:
                    return False
                self.slice_from("\u062F")
                self.B_remove_verb_person_endings = True
            else:
                if self.cursor <= self.limit_backward:
                    return False
                self.slice_from("\u062A")
                self.B_remove_verb_person_endings = True
            break
        return True

    def _stem(self):
        self.B_saw_present_prefix = False
        v_1 = self.cursor
        self.__r_Normalize_Characters()
        self.cursor = v_1
        v_2 = self.cursor
        self.__r_Prefixes()
        self.cursor = v_2
        v_3 = self.cursor
        self.__r_Delete_ZWNJ()
        self.cursor = v_3
        self.I_p1 = self.limit
        v_4 = self.cursor
        try:
            if self.cursor + 3 > self.limit:
                raise lab0()
            self.cursor += 3
            self.I_p1 = self.cursor
        except lab0: pass
        self.cursor = v_4
        self.limit_backward = self.cursor
        self.cursor = self.limit
        while True:
            v_5 = self.limit - self.cursor
            try:
                v_6 = self.limit - self.cursor
                self.B_remove_verb_person_endings = False
                try:
                    if not self.B_saw_present_prefix:
                        raise lab1()
                    self.B_remove_verb_person_endings = True
                except lab1: pass
                if not self.__r_Protect_Lexical_AN():
                    raise lab0()
                while True:
                    v_7 = self.limit - self.cursor
                    try:
                        if not self.__r_Stem_Noun_or_Adjective():
                            raise lab1()
                        break
                    except lab1: pass
                    self.cursor = self.limit - v_7
                    if not self.__r_Stem_Verb():
                        raise lab0()
                    break
                self.cursor = self.limit - v_6
                continue
            except lab0: pass
            self.cursor = self.limit - v_5
            break
        self.cursor = self.limit_backward
        return True

    a_0 = [
        Among("", -1, 7),
        Among(" ", 0, 6),
        Among("\u0623", 0, 4),
        Among("\u0624", 0, 5),
        Among("\u0625", 0, 4),
        Among("\u0626", 0, 2),
        Among("\u0629", 0, 3),
        Among("\u0643", 0, 1),
        Among("\u064A", 0, 2),
        Among("\u06C1", 0, 3),
        Among("\u200D", 0, 6)
    ]

    a_1 = [
        Among("\u0645\u06CC\u200C", -1, 2),
        Among("\u0646\u0645\u06CC\u200C", -1, 1)
    ]

    a_2 = [
        Among("\u0633\u062A\u0627\u0646", -1, -1),
        Among("\u0631\u0627\u0646", -1, -1),
        Among("\u0633\u0627\u0646", -1, -1),
        Among("\u0648\u0627\u0646", -1, -1)
    ]

    a_3 = [
        Among("\u0622\u0630\u0631\u0628\u0627\u06CC\u062C\u0627\u0646", -1, 1),
        Among("\u0647\u0645\u062F\u0627\u0646", -1, 1),
        Among("\u062E\u0627\u0646\u062F\u0627\u0646", -1, 1),
        Among("\u0632\u0646\u062F\u0627\u0646", -1, 1),
        Among("\u0645\u06CC\u0632\u0627\u0646", -1, 1),
        Among("\u062F\u0631\u062E\u0634\u0627\u0646", -1, 1),
        Among("\u0622\u062A\u0634\u0641\u0634\u0627\u0646", -1, 1),
        Among("\u0646\u0634\u0627\u0646", -1, 1),
        Among("\u06A9\u0647\u06A9\u0634\u0627\u0646", -1, 1),
        Among("\u0627\u06CC\u0634\u0627\u0646", -1, 1),
        Among("\u067E\u0631\u06CC\u0634\u0627\u0646", -1, 1),
        Among("\u0633\u0644\u0637\u0627\u0646", -1, 1),
        Among("\u06AF\u06CC\u0644\u0627\u0646", -1, 1),
        Among("\u0633\u0627\u062E\u062A\u0645\u0627\u0646", -1, 1),
        Among("\u0631\u0645\u0627\u0646", -1, 1),
        Among("\u062F\u0631\u0645\u0627\u0646", 14, 1),
        Among("\u0642\u0647\u0631\u0645\u0627\u0646", 14, 1),
        Among("\u06A9\u0631\u0645\u0627\u0646", 14, 1),
        Among("\u0633\u0627\u0632\u0645\u0627\u0646", -1, 1),
        Among("\u0647\u0645\u0632\u0645\u0627\u0646", -1, 1),
        Among("\u0622\u0633\u0645\u0627\u0646", -1, 1),
        Among("\u0622\u0644\u0645\u0627\u0646", -1, 1),
        Among("\u0645\u0633\u0644\u0645\u0627\u0646", -1, 1),
        Among("\u0627\u06CC\u0645\u0627\u0646", -1, 1),
        Among("\u0633\u0644\u06CC\u0645\u0627\u0646", -1, 1),
        Among("\u067E\u06CC\u0645\u0627\u0646", -1, 1),
        Among("\u0644\u0628\u0646\u0627\u0646", -1, 1),
        Among("\u06CC\u0648\u0646\u0627\u0646", -1, 1),
        Among("\u0627\u0635\u0641\u0647\u0627\u0646", -1, 1),
        Among("\u0627\u0645\u06A9\u0627\u0646", -1, 1),
        Among("\u067E\u0627\u06CC\u0627\u0646", -1, 1),
        Among("\u0628\u06CC\u0627\u0646", -1, 1),
        Among("\u062C\u0631\u06CC\u0627\u0646", -1, 1)
    ]

    a_4 = [
        Among("\u0627\u0633\u0627\u062A\u06CC\u062F", -1, 2),
        Among("\u0627\u062E\u0628\u0627\u0631", -1, 1)
    ]
    as_4 = ("\u062E\u0628\u0631", "\u0627\u0633\u062A\u0627\u062F")

    a_5 = [
        Among("\u0647\u0627", -1, 1),
        Among("\u0627\u062A", -1, 1),
        Among("\u06CC\u062A", -1, 1),
        Among("\u0645\u0646\u062F", -1, 1),
        Among("\u0648\u0627\u0631", -1, 1),
        Among("\u06AF\u0627\u0631", -1, 1),
        Among("\u062A\u0631", -1, 2),
        Among("\u0627\u0634", -1, 1),
        Among("\u0627\u0645", -1, 1),
        Among("\u0627\u0646", -1, 1),
        Among("\u0628\u0627\u0646", 9, 1),
        Among("\u06AF\u0627\u0646", 9, 1),
        Among("\u06CC\u0627\u0646", 9, 1),
        Among("\u06CC\u0646", -1, 1),
        Among("\u062A\u0631\u06CC\u0646", 13, 1),
        Among("\u06AF\u0627\u0647", -1, 1),
        Among("\u0627\u0646\u0647", -1, 1),
        Among("\u0646\u0627\u06A9", -1, 1),
        Among("\u0647\u0627\u06CC", -1, 1),
        Among("\u0627\u0646\u06CC", -1, 1),
        Among("\u06AF\u06CC", -1, 1),
        Among("\u06CC\u06CC", -1, 1)
    ]

    a_6 = [
        Among("\u0627\u0633\u062A", -1, 1),
        Among("\u0627\u0646\u062F", -1, 1),
        Among("\u06CC\u062F", -1, 1),
        Among("\u0627\u06CC\u062F", 2, 1),
        Among("\u0627\u0633", -1, 1),
        Among("\u06CC\u0645", -1, 1),
        Among("\u0627\u06CC\u0645", 5, 1),
        Among("\u0627\u06CC", -1, 1)
    ]

    a_7 = [
        Among("\u062F", -1, 1),
        Among("\u0627\u0646\u062F", 0, 1),
        Among("\u0631\u0641\u062A\u0627\u0646\u062F", 1, 2),
        Among("\u06CC\u062F", 0, 1),
        Among("\u0631\u0641\u062A\u06CC\u062F", 3, 2),
        Among("\u0645", -1, 1),
        Among("\u0627\u0645", 5, 1),
        Among("\u0631\u0641\u062A\u0645", 5, 2),
        Among("\u06CC\u0645", 5, 1),
        Among("\u0631\u0641\u062A\u06CC\u0645", 8, 2),
        Among("\u0627\u0646", -1, 3),
        Among("\u062A\u0647", -1, 5),
        Among("\u062F\u0647", -1, 4),
        Among("\u0646\u062F\u0647", 12, 3),
        Among("\u0631\u0641\u062A\u06CC", -1, 2)
    ]


class lab0(BaseException): pass


class lab1(BaseException): pass



# Generated from danish.sbl by Snowball 3.1.1 - https://snowballstem.org/

from .basestemmer import BaseStemmer
from .among import Among


class DanishStemmer(BaseStemmer):
    '''
    This class implements the stemming algorithm defined by a snowball script.
    Generated from danish.sbl by Snowball 3.1.1 - https://snowballstem.org/
    '''

    g_undouble_c = {"b", "d", "f", "g", "k", "l", "m", "n", "p", "r", "s", "t"}

    g_v = {"a", "e", "i", "o", "u", "y", "å", "æ", "ø"}

    g_s_ending = {"'", "a", "b", "c", "d", "f", "g", "h", "j", "k", "l", "m", "n", "o", "p", "r", "t", "v", "y", "z", "å"}

    I_p1 = 0

    def __r_mark_regions(self):
        self.I_p1 = self.limit
        v_1 = self.cursor
        try:
            while True:
                v_2 = self.cursor
                try:
                    while True:
                        try:
                            if self.cursor == self.limit or self.current[self.cursor] != "'":
                                raise lab2()
                            self.cursor += 1
                            break
                        except lab2: pass
                        if self.cursor >= self.limit:
                            raise lab1()
                        self.cursor += 1
                    break
                except lab1: pass
                self.cursor = v_2
                if not self.go_out_grouping(DanishStemmer.g_v):
                    raise lab0()
                self.cursor += 1
                if not self.go_in_grouping(DanishStemmer.g_v):
                    raise lab0()
                self.cursor += 1
                break
            self.I_p1 = self.cursor
        except lab0: pass
        self.cursor = v_1
        v_3 = self.cursor
        if self.cursor + 3 > self.limit:
            return False
        self.cursor += 3
        try:
            if self.I_p1 >= self.cursor:
                raise lab0()
            self.I_p1 = self.cursor
        except lab0: pass
        self.cursor = v_3
        return True

    def __r_main_suffix(self):
        if self.cursor < self.I_p1:
            return False
        v_2 = self.limit_backward
        self.limit_backward = self.I_p1
        self.ket = self.cursor
        among_var = self.find_among_b(DanishStemmer.a_0)
        if among_var == 0:
            self.limit_backward = v_2
            return False
        self.bra = self.cursor
        self.limit_backward = v_2
        if among_var == 1:
            self.slice_del()
        else:
            if not self.in_grouping_b(DanishStemmer.g_s_ending):
                return False
            self.slice_del()
        return True

    def __r_consonant_pair(self):
        v_1 = self.limit - self.cursor
        if self.cursor < self.I_p1:
            return False
        v_3 = self.limit_backward
        self.limit_backward = self.I_p1
        self.ket = self.cursor
        if self.find_among_b(DanishStemmer.a_1) == 0:
            self.limit_backward = v_3
            return False
        self.bra = self.cursor
        self.limit_backward = v_3
        self.cursor = self.limit - v_1
        if self.cursor <= self.limit_backward:
            return False
        self.cursor -= 1
        self.bra = self.cursor
        self.slice_del()
        return True

    def __r_other_suffix(self):
        v_1 = self.limit - self.cursor
        try:
            self.ket = self.cursor
            if not self.eq_s_b("st"):
                raise lab0()
            self.bra = self.cursor
            if not self.eq_s_b("ig"):
                raise lab0()
            self.slice_del()
        except lab0: pass
        self.cursor = self.limit - v_1
        if self.cursor < self.I_p1:
            return False
        v_3 = self.limit_backward
        self.limit_backward = self.I_p1
        self.ket = self.cursor
        among_var = self.find_among_b(DanishStemmer.a_2)
        if among_var == 0:
            self.limit_backward = v_3
            return False
        self.bra = self.cursor
        self.limit_backward = v_3
        if among_var == 1:
            self.slice_del()
            v_4 = self.limit - self.cursor
            self.__r_consonant_pair()
            self.cursor = self.limit - v_4
        else:
            self.slice_from("løs")
        return True

    def __r_undouble(self):
        if self.cursor < self.I_p1:
            return False
        v_2 = self.limit_backward
        self.limit_backward = self.I_p1
        self.ket = self.cursor
        if not self.in_grouping_b(DanishStemmer.g_undouble_c):
            self.limit_backward = v_2
            return False
        self.bra = self.cursor
        S_ch = self.slice_to()
        self.limit_backward = v_2
        if not self.eq_s_b(S_ch):
            return False
        self.slice_del()
        return True

    def _stem(self):
        if not self.__r_mark_regions():
            return False
        self.limit_backward = self.cursor
        self.cursor = self.limit
        v_1 = self.limit - self.cursor
        self.__r_main_suffix()
        self.cursor = self.limit - v_1
        v_2 = self.limit - self.cursor
        self.__r_consonant_pair()
        self.cursor = self.limit - v_2
        v_3 = self.limit - self.cursor
        self.__r_other_suffix()
        self.cursor = self.limit - v_3
        v_4 = self.limit - self.cursor
        self.__r_undouble()
        self.cursor = self.limit - v_4
        self.ket = self.cursor
        if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "'":
            return False
        self.cursor -= 1
        self.bra = self.cursor
        self.slice_del()
        self.cursor = self.limit_backward
        return True

    a_0 = [
        Among("hed", -1, 1),
        Among("ethed", 0, 1),
        Among("ered", -1, 1),
        Among("e", -1, 1),
        Among("erede", 3, 1),
        Among("ende", 3, 1),
        Among("erende", 5, 1),
        Among("ene", 3, 1),
        Among("erne", 3, 1),
        Among("ere", 3, 1),
        Among("en", -1, 1),
        Among("heden", 10, 1),
        Among("eren", 10, 1),
        Among("er", -1, 1),
        Among("heder", 13, 1),
        Among("erer", 13, 1),
        Among("s", -1, 2),
        Among("heds", 16, 1),
        Among("es", 16, 1),
        Among("endes", 18, 1),
        Among("erendes", 19, 1),
        Among("enes", 18, 1),
        Among("ernes", 18, 1),
        Among("eres", 18, 1),
        Among("ens", 16, 1),
        Among("hedens", 24, 1),
        Among("erens", 24, 1),
        Among("ers", 16, 1),
        Among("ets", 16, 1),
        Among("erets", 28, 1),
        Among("et", -1, 1),
        Among("eret", 30, 1)
    ]

    a_1 = [
        Among("gd", -1, -1),
        Among("dt", -1, -1),
        Among("gt", -1, -1),
        Among("kt", -1, -1)
    ]

    a_2 = [
        Among("ig", -1, 1),
        Among("lig", 0, 1),
        Among("elig", 1, 1),
        Among("els", -1, 1),
        Among("løst", -1, 2)
    ]


class lab0(BaseException): pass


class lab1(BaseException): pass


class lab2(BaseException): pass
